# -*- coding: utf-8 -*-
"""Data manipulation tools.

"""


from glob import glob
import collections
import itertools as it

import numpy as np
import pandas as pd


# Split an iterable by equal length
__all__ = ['exc',
           'splitter',
           'even_chunk',
           'pair_unique',
           'df_pair_unique',
           'map_to_tuple',
           'map_to_list',
           'merge_csv',
           'nancumsum',
           'depth',
           'zero_padder_2d',
           'zero_padder_3d']


def exc(source, blacklist):
    """Get items except the given list.

    This function splits an Iterable into the given size of multiple chunks.
    The items of An iterable should be the same type.

    Parameters
    ----------
    source: Iterable
        An Iterable to filter.

    blacklist: Iterable
        A list contains items to eliminate.

    Returns
    -------
    list
        A filtered list.

    See Also
    --------
    ``Infix Operator``

    Examples
    --------
    >>> import unipy as up
    >>> up.splitter(list(range(10)), how='equal', size=3)
    [(0, 1, 2, 3), (4, 5, 6), (7, 8, 9)]

    >>> up.splitter(list(range(10)), how='remaining', size=3)
    [(0, 1, 2), (3, 4, 5), (6, 7, 8), (9,)]

    """
    res = [item for item in source if item not in blacklist]

    return res


# A Function to split an Iterable into smaller chunks
def splitter(iterable, how='equal', size=2):
    """Split data with given size.

    This function splits an Iterable into the given size of multiple chunks.
    The items of An iterable should be the same type.

    Parameters
    ----------
    iterable: Iterable
        An Iterable to split.

    how: {'equal', 'remaining'}
        The method to split.
        'equal' is to split chunks with the approximate length
        within the given size.
        'remaining' is to split chunks with the given size,
        and the remains are bound as the last chunk.

    size: int
        The number of chunks.

    Returns
    -------
    list
        A list of chunks.

    See Also
    --------
    numpy.array_split
    itertools.islice

    Examples
    --------
    >>> import unipy as up
    >>> up.splitter(list(range(10)), how='equal', size=3)
    [(0, 1, 2, 3), (4, 5, 6), (7, 8, 9)]

    >>> up.splitter(list(range(10)), how='remaining', size=3)
    [(0, 1, 2), (3, 4, 5), (6, 7, 8), (9,)]

    """
    isinstance(iterable, collections.Iterable)
    isinstance(size, int)

    if not size > 0:
        raise ValueError("'size' must be greater than 0")
    else:
        if how == 'equal':
            splitted = np.array_split(iterable, (len(iterable) / size) + 1)
            resList = [tuple(chunks) for chunks in splitted]
            return resList

        elif how == 'remaining':
            tmpIterator = iter(iterable)
            splitted = iter(lambda: tuple(it.islice(tmpIterator, size)), ())
            resList = list(splitted)
            return resList


def _even_chunk(iterable, chunk_size):
    assert isinstance(iterable, collections.Iterable)
    iterator = iter(iterable)
    slicer = iter(lambda: list(it.islice(iterator, chunk_size)), [])
    yield from slicer


def _even_chunk_arr(arr, chunk_size, axis=0):
    assert isinstance(arr, np.ndarray)
    if axis in [0, 'row']:
        slicer = _even_chunk(arr, chunk_size)
    elif axis in [1, 'column']:
        slicer = _even_chunk(arr.T, chunk_size)
    return slicer


def _even_chunk_df(df, chunk_size, axis=0):
    assert isinstance(df, pd.DataFrame)
    if axis in [0, 'row']:
        colnames = df.columns
        zipped = zip(_even_chunk(df.index, chunk_size),
                     _even_chunk(df.values, chunk_size))
        slicer = (pd.DataFrame(row_arr, index=row_idx, columns=colnames)
                  for row_idx, row_arr in zipped)
    elif axis in [1, 'column']:
        rownames = df.index
        zipped = zip(_even_chunk(df.columns, chunk_size),
                     _even_chunk(df.values.T, chunk_size))
        slicer = (pd.DataFrame(col_arr, index=col_idx, columns=rownames).T
                  for col_idx, col_arr in zipped)

    yield from slicer


def _even_chunk_series(series, chunk_size):
    assert isinstance(series, pd.Series)

    name = series.name
    zipped = zip(_even_chunk(series.index, chunk_size),
                 _even_chunk(series.values, chunk_size))
    slicer = (pd.Series(val_arr, index=idx, name=name)
              for idx, val_arr in zipped)

    yield from slicer


def even_chunk(iterable, chunk_size, *args, **kwargs):
    """Split data into even size.

    This function splits an Iterable into the given size of multiple chunks.
    The items of An iterable should be the same type.

    Parameters
    ----------
    iterable: Iterable
        An Iterable to split. If N-dimensional, It is chunked by 1st dimension.

    chunk_size: ``int``
        The length of each chunks.

    Returns
    -------
    generator
        A generator yields a list of chunks. The data type of the elements
        in a list are equal to the source data type.

    See Also
    --------
    ``itertools.islice``
    ``yield from``

    Examples
    --------
    >>> import numpy as np
    >>> from unipy.tools.data_handler import even_chunk
    >>> data = list(range(7))  # list, 1D
    >>> print(data)
    [0, 1, 2, 3, 4, 5, 6]
    >>> chunked_gen = even_chunk(data, 3)
    >>> print(chunked_gen)
    <generator object even_chunk at 0x7fc4924897d8>
    >>> next(chunked_gen)
    [0, 1, 2]
    >>> chunked = list(even_chunk(data, 3))
    >>> print(chunked)
    [[0, 1, 2], [3, 4, 5], [6]]
    >>> data = np.arange(30).reshape(-1, 3)  # np.ndarray, 2D
    >>> print(data)
    array([[ 0,  1,  2],
           [ 3,  4,  5],
           [ 6,  7,  8],
           [ 9, 10, 11],
           [12, 13, 14],
           [15, 16, 17],
           [18, 19, 20],
           [21, 22, 23],
           [24, 25, 26],
           [27, 28, 29]])
    >>> chunked_gen = even_chunk(data, 4)
    >>> next(chunked_gen)
    [array([0, 1, 2]), array([3, 4, 5]), array([6, 7, 8]), array([ 9, 10, 11])]
    >>> next(chunked_gen)
    [array([12, 13, 14]),
     array([15, 16, 17]),
     array([18, 19, 20]),
     array([21, 22, 23])]
    >>> next(chunked_gen)
    [array([24, 25, 26]), array([27, 28, 29])]
    >>> next(chunked_gen)
    Traceback (most recent call last):
    File "<stdin>", line 1, in <module>
    StopIteration

    """
    if isinstance(iterable, np.ndarray):
        chunked = _even_chunk_arr(iterable, chunk_size, *args, **kwargs)
    elif isinstance(iterable, pd.DataFrame):
        chunked = _even_chunk_df(iterable, chunk_size, *args, **kwargs)
    elif isinstance(iterable, pd.Series):
        chunked = _even_chunk_series(iterable, chunk_size)
    else:
        chunked = _even_chunk(iterable, chunk_size)

    return chunked


def pair_unique(*args):
    """Get Unique pairsets.

    This function gets an unique pair-sets of given data.

    Parameters
    ----------
    iterable: Iterable
        Iterables having an equal length.

    Returns
    -------
    list
        A list of tuples. Each tuple is an unique pair of values.

    Raises
    ------
    ValueError
        If the lengths of argments are not equal.

    See Also
    --------
    ``zip``
    ``set``

    Examples
    --------
    >>> from unipy.tools.data_handler import pair_unique
    >>> data = dm.load('titanic')
    Dataset : titanic
    >>> data.head()
      Class     Sex    Age Survived  Freq
    0   1st    Male  Child       No     0
    1   2nd    Male  Child       No     0
    2   3rd    Male  Child       No    35
    3  Crew    Male  Child       No     0
    4   1st  Female  Child       No     0
    >>> pair_unique(data.iloc[:, 0], data.iloc[:, 1])
    [(5, '1st'), (19, '3rd'), (29, '1st'), (20, 'Crew'),
     (21, '1st'), (3, '3rd'), (16, 'Crew'), (26, '2nd'),
     (23, '3rd'), (10, '2nd'), (24, 'Crew'), (7, '3rd'),
     (4, 'Crew'), (27, '3rd'), (18, '2nd'), (28, 'Crew'),
     (30, '2nd'), (11, '3rd'), (2, '2nd'), (1, '1st'),
     (14, '2nd'), (31, '3rd'), (22, '2nd'), (17, '1st'),
     (8, 'Crew'), (9, '1st'), (32, 'Crew'), (15, '3rd'),
     (6, '2nd'), (12, 'Crew'), (13, '1st'), (25, '1st')]
    >>> idx1 = [1, 2, 3]
    >>> idx2 = [0, 9, 8, 4]
    >>> pair_unique(idx1, idx2)
    Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
    ValueError: All argments should have the same length.

    """
    args_tuple = (*args, )
    for x in range(len(args_tuple)):
        isinstance(x, collections.Iterable)
    len_args = [len(arg) for arg in args_tuple]
    if len(set(len_args)) != 1:
        raise ValueError('All argments should have the same length.')

    res_list = list(set(zip(*args)))

    return res_list


# Unique Pair List Creator For DataFrame
def df_pair_unique(data_frame, col_list, to_frame=False):
    """Get unique pairsets in pandas.DataFrame.

    This function gets an unique pair-sets of given columns.

    Parameters
    ----------
    data_frame: pandas.DataFrame
        DataFrame to get unique-pairs.

    col_list: pandas.Index, list, tuple
        Column names of given DataFrame.

    to_frame: Boolean (default: False)
        Choose output type.
        If True, It returns pandas.DataFrame as an output.
        If False, It returns a list of tuples.

    Returns
    -------
    list
        If to_frame=False, A list of tuples is returned.
        Each tuple is an unique pair of values.

    pandas.DataFrame
        If to_frame=True, pandas.DataFrame is returned.
        Each row is an unique pair of values.

    See Also
    --------
    pandas.DataFrame.itertuples

    Examples
    --------
    >>> from unipy.tools.data_handler import df_pair_unique
    >>> data = dm.load('titanic')
    Dataset : titanic
    >>> data.head()
      Class     Sex    Age Survived  Freq
    0   1st    Male  Child       No     0
    1   2nd    Male  Child       No     0
    2   3rd    Male  Child       No    35
    3  Crew    Male  Child       No     0
    4   1st  Female  Child       No     0
    >>> df_pair_unique(data, ['Class', 'Sex'])
    [('3rd', 'Male'), ('2nd', 'Male'), ('2nd', 'Female'), ('1st', 'Female'),
     ('Crew', 'Male'), ('1st', 'Male'), ('Crew', 'Female'), ('3rd', 'Female')]
    >>> df_pair_unique(data, ['Class', 'Sex'], to_frame=True)
      Class     Sex
    0   3rd    Male
    1   2nd    Male
    2   2nd  Female
    3   1st  Female
    4  Crew    Male
    5   1st    Male
    6  Crew  Female
    7   3rd  Female

    """
    args_tuple_map = data_frame[col_list].itertuples(index=False)
    res_list = list(set(tuple(idx) for idx in args_tuple_map))
    if to_frame:
        return pd.DataFrame(res_list, columns=col_list)
    else:
        return res_list


# %% Item Transformator
def map_to_tuple(iterable):
    """Only for some specific reason.

    """
    isinstance(iterable, collections.Iterable)
    for item in iterable:
        isinstance(item, collections.Iterator)
    res = tuple(map(lambda item: tuple(item), iterable))

    return res


def map_to_list(iterable):
    """Only for some specific reason.

    """
    isinstance(iterable, collections.Iterable)
    for item in iterable:
        isinstance(item, collections.Iterator)
    res = list(map(lambda item: tuple(item), iterable))

    return res


# %% Data Concatenator within a Folder
def merge_csv(file_path, pattern='*.csv', sep=',',
              if_save=True, save_name=None, low_memory=True):
    """Merge seperated csv type datasets into one dataset.
    Summary

    This function get separated data files together.
    When merged, the file is sorted by its name in ascending order.

    Parameters
    ----------
    file_path: str
        A directory path of source files.

    pattern: str
        A File extension with conditional naming. (default: '*.csv')

    sep: int
        A symbol seperating data columns.

    if_save: Boolean (Optional, default: True)
        False if you don't want to save the result.

    save_name: str
        A filename to save the result.
        It should be given if if_save=True.
        If inappropriate name is given, the first name of file list is used.

    low_memory: Boolean (Optional, default: True)
        It is used for pandas.read_csv() option only.

    Returns
    -------
    pandas.DataFrame
        A concatenated DataFrame.

    See Also
    --------

    Examples
    --------
    >>> from unipy.tools.data_handler import merge_csv
    >>> data = dm.load('titanic')
    Dataset : titanic
    >>> data.head(9)
      Class     Sex    Age Survived  Freq
    0   1st    Male  Child       No     0
    1   2nd    Male  Child       No     0
    2   3rd    Male  Child       No    35
    3  Crew    Male  Child       No     0
    4   1st  Female  Child       No     0
    5   2nd  Female  Child       No     0
    6   3rd  Female  Child       No    17
    7  Crew  Female  Child       No     0
    8   1st    Male  Adult       No   118
    >>> data.iloc[:2, :].to_csv('tmp1.csv', header=True, index=False)
    >>> data.iloc[2:4, :].to_csv('tmp2.csv', header=True, index=False)
    >>> data.iloc[4:9, :].to_csv('tmp3.csv', header=True, index=False)
    >>> merged = merge_csv('./')
    >>> merged
      Class     Sex    Age Survived  Freq
    0   1st    Male  Child       No     0
    1   2nd    Male  Child       No     0
    2   3rd    Male  Child       No    35
    3  Crew    Male  Child       No     0
    4   1st  Female  Child       No     0
    5   2nd  Female  Child       No     0
    6   3rd  Female  Child       No    17
    7  Crew  Female  Child       No     0
    8   1st    Male  Adult       No   118

    """
    if file_path[-1] != '/':
        file_path = file_path + '/'
    file_list = sorted(glob(file_path + pattern))

    res_frame = pd.DataFrame()
    for filename in file_list:
        file = pd.read_csv(filename, sep=sep, low_memory=low_memory)
        res_frame = res_frame.append(file, ignore_index=True)
    print('Concat Compelete.')

    if if_save:
        save_msg = "Saving it to '{save_name}'"
        if isinstance(save_name, str):
            save_name = file_path + save_name
            print(save_msg.format(save_name=save_name))
        else:
            sample_name = file_list[0].split('/')[-1]
            ext = '.' + sample_name.split('.')[-1]
            save_name = sample_name[:-(len(ext))] + '_concat' + ext
            print("'save_name' is inappropriate: '{}'".format(save_name))
            print(save_msg.format(save_name=save_name))

        res_frame.to_csv(save_name, header=True, index=False)

    return res_frame


def nancumsum(iterable):
    """A cumulative sum function.

    A cumulative sum function.

    Parameters
    ----------
    iterable: Iterable
        Iterables to calculate cumulative sum.

    Yields
    -------
    int
        A cumulative summed value.

    See Also
    --------
    numpy.isnan

    Examples
    --------
    >>> from unipy.tools.data_handler import nancumsum
    >>> tmp = [1, 2, 4]
    >>> nancumsum(tmp)
    <generator object nancumsum at 0x1084553b8>
    >>> list(nancumsum(tmp))
    [1, 3, 7]

    """
    iterator = iter(iterable)
    prev = next(iterator)
    yield prev

    for item in iterator:
        if ~np.isnan(item):
            res = prev + item
        prev = res
        yield res


def depth(iterable):
    """Get dimension depth.

    Get a dimension depth number of a nested iterable.

    Parameters
    ----------
    iterable: iterable
        An Iterable to get a dimension depth number.

    Returns
    -------
    int
        A dimension depth number.

    See Also
    --------
    collections.Iterable

    Examples
    --------
    >>> from unipy.tools.data_handler import depth
    >>> tmp = [(1, 3), (4, 6), (7, 9), (10, 12)]
    >>> depth(tmp)
    2
    >>> tmp3d = [[np.arange(i) + i for i in range(2, j)]
    ...          for j in range(5, 10)]
    >>> depth(tmp3d)
    3
    >>> # It can handle dict type (considering values only).
    >>> tmp3d_dict = [{'key' + str(i): np.arange(i) + i for i in range(2, j)}
    ...               for j in range(5, 10)]
    >>> depth(tmp3d_dict)
    3

    """
    assert isinstance(iterable, collections.Iterable), 'Not an Iterable.'
    container = iterable
    depth = 0
    while True:
        try:
            if isinstance(container, dict):
                container = iter(container.values())
            elif isinstance(container, str):
                depth += 1
                raise TypeError
            else:
                container = iter(container)
            element = next(container)
            depth += 1
            container = element

        except TypeError:
            break

    return depth


def zero_padder_2d(arr, max_len=None, method='backward'):
    """Zero-padding for fixed-length inputs(2D).

    Zero-padding Function with nested sequence.
    Each elements of a given sequence is padded fixed-length.

    Parameters
    ----------
    arr: Iterable
        A nested sequence containing 1-Dimensional numpy.ndarray.

    max_len: int (default: None)
        A required fixed-length of each sequences.
        If None, It calculates the max length of elements as max_len.

    method: {'forward', 'backward'} (default: 'backward')
        where to pad.

    Returns
    -------
    list
        A list containing 3-Dimensional numpy.ndarray with fixed-length 2D.

    See Also
    --------
    unipy.depth
    numpy.pad
    numpy.stack

    Examples
    --------
    >>> from unipy.tools.data_handler import zero_padder_2d
    >>> tmp = [np.arange(i) + i for i in range(2, 5)]
    >>> tmp
    [array([2, 3]), array([3, 4, 5]), array([4, 5, 6, 7])]
    >>> zero_padder_2d(tmp)
    array([[2, 3, 0, 0],
           [3, 4, 5, 0],
           [4, 5, 6, 7]])
    >>> zero_padder_2d(tmp, max_len=6)
    array([[2, 3, 0, 0, 0, 0],
           [3, 4, 5, 0, 0, 0],
           [4, 5, 6, 7, 0, 0]])
     >>> zero_padder_2d(tmp, max_len=5, method='forward')
    array([[0, 0, 0, 2, 3],
           [0, 0, 3, 4, 5],
           [0, 4, 5, 6, 7]])

    """
    assert isinstance(arr, collections.Iterable)
    assert all(isinstance(item, collections.Iterable) for item in arr)
    assert depth(arr) == 2, 'Not 2-Dimensional.'
    assert method in ['forward', 'backward']
    arr_max_len = max(map(len, arr))
    if max_len is None:
        max_len = arr_max_len
    else:
        assert max_len >= arr_max_len
    if method == 'forward':
        res = [np.pad(item, (max_len-len(item), 0),
                      mode='constant',
                      constant_values=0)
               for item in arr]
    elif method == 'backward':
        res = [np.pad(item, (0, max_len-len(item)),
                      mode='constant',
                      constant_values=0)
               for item in arr]
    res = np.stack(res)
    # return np.asarray(res)
    return np.stack(res)


def zero_padder_3d(arr, max_len=None, method='backward'):
    """Zero-padding for fixed-length inputs(3D).

    Zero-padding Function with nested sequence.
    Each elements of a given sequence is padded fixed-length.

    Parameters
    ----------
    arr: Iterable
        A nested sequence containing 2-Dimensional numpy.ndarray.

    max_len: int (default: None)
        A required fixed-length of each sequences.
        If None, It calculates the max length of elements as max_len.

    method: {'forward', 'backward'} (default: 'backward')
        where to pad.

    Returns
    -------
    list
        A list containing 3-Dimensional numpy.ndarray with fixed-length 2D.

    Raises
    ------
    ValueError
        All 3D shape of inner numpy.ndarray is not equal.

    See Also
    --------
    unipy.depth
    numpy.pad
    numpy.stack

    Examples
    --------
    >>> from unipy.tools.data_handler import zero_padder_3d
    >>> tmp3d = [np.arange(i * 2).reshape(-1, 2) for i in range(1, 5)]
    >>> tmp3d
    [array([[0, 1]]),
     array([[0, 1],
            [2, 3]]),
     array([[0, 1],
            [2, 3],
            [4, 5]]),
     array([[0, 1],
            [2, 3],
            [4, 5],
            [6, 7]])]
    >>> zero_padder_3d(tmp3d)
    array([[[0, 1],
            [0, 0],
            [0, 0],
            [0, 0]],

           [[0, 1],
            [2, 3],
            [0, 0],
            [0, 0]],

           [[0, 1],
            [2, 3],
            [4, 5],
            [0, 0]],

           [[0, 1],
            [2, 3],
            [4, 5],
            [6, 7]]])
    >>> tmp3d_eye
    [array([[1.]]),
     array([[1., 0.],
            [0., 1.]]),
     array([[1., 0., 0.],
            [0., 1., 0.],
            [0., 0., 1.]]),
     array([[1., 0., 0., 0.],
            [0., 1., 0., 0.],
            [0., 0., 1., 0.],
            [0., 0., 0., 1.]])]
    >>> zero_padder_3d(tmp3d_eye)
    Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
      File "<stdin>", line 24, in zero_padder_3d
    ValueError: 3D shape should be equal.

    """
    assert isinstance(arr, collections.Iterable)
    assert all(isinstance(item, np.ndarray) for item in arr)
    assert depth(arr) == 3, 'Not 3-Dimensional.'
    assert method in ['forward', 'backward']
    arr_max_len = max(map(len, arr))
    if max_len is None:
        max_len = arr_max_len
    else:
        assert max_len >= arr_max_len
    try:
        if method == 'forward':
            res = [np.pad(item, ((max_len-len(item), 0), (0, 0)),
                          mode='constant',
                          constant_values=0)
                   for item in arr]
        elif method == 'backward':
            res = [np.pad(item, ((0, max_len-len(item)), (0, 0)),
                          mode='constant',
                          constant_values=0)
                   for item in arr]
        return np.stack(res)
    except ValueError:
        raise ValueError('3D shape should be equal.')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jul 22 23:43:37 2017

@author: pydemia
"""


if __name__ == '__main__':
    import numpy as np
    import pandas as pd
    import scipy.stats as st
    import statsmodels
    import statsmodels.api as sm
    import sklearn as skl
    import unipy
    import unipy.dataset.api as dm

    import matplotlib
    matplotlib.use('Agg')

    import matplotlib.pyplot as plt
    import seaborn as sbn


    dm.init()
    data01 = dm.load('iris')
    data02 = data01.iloc[:, :2]

    col01 = data01['sepal_length']
    col02 = data01['sepal_width']
    col03 = data01['petal_length']
    col04 = data01['petal_width']


    data11 = dm.load('adult')
    data11.columns
    data11

    col11 = data11['education']
    col12 = data11['race']


    #data11 = dm.load('titanic')
    #data11.columns
    #data11
    #dm.ls()
    #col11 = data11['Class']
    #col12 = data11['Survived']

    # %% T-test

    import sklearn as skl

    # 1 sample
    t_stat, pvalue = st.ttest_1samp(col01, col01.mean(), axis=0, nan_policy='propagate')

    # 2 independents have identical averages.
    # (This test assumes that the populations have identical variances.)
    st.ttest_ind(col01, col02, axis=0, equal_var=True)
    sm.stats.ttest_ind(col01, col02)

    #plt.hist(col01, bins=None)
    #plt.hist(col02, bins=None)
    #col01.plot(kind='density')
    #data02.plot(kind='density')
    #sbn.distplot(col01)
    #sbn.kdeplot(col01)

    # check 2 related or repeated samples have identical averages.
    # Defines how to handle when input contains nan.
    # ‘propagate’ returns nan
    # ‘raise’ throws an error
    # ‘omit’ performs the calculations ignoring nan values
    st.ttest_rel(col01, col02, axis=0, nan_policy='propagate')

    #data02.plot(kind='density')


    # %% KS-Test
    # Kolmogorov-Smirnov test for goodness of fit
    st.kstest('norm', False, N=20, alternative='two-sided', mode='approx')


    # %% Chi-Square Test
    # One-way X2 test

    st.chisquare(col01, col02)
    st.chi2_contingency(col01, col02)
    #st.chi2_contingency(col11, col12)

    unipy.chisq_test(data11, x='education', y='race')

    # %% Bartlett Test
    # Test for equal variances

    st.bartlett(col01, col02)

    # %% Levene Test

    st.levene(col01, col02, center='median')

    # %% One-way ANOVA

    st.f_oneway(col01, col02, col03)

    # %% Lasso


    #unipy.lasso(data01.iloc[:, :-1])



"""Docstring for ``decorator``.

========================
Function Decorator
========================
==================== =========================================================
File Transfer
==============================================================================
gdrive_downloader    File downloader from Google Drive.
gdrive_uploader      File uploader to Google Drive.
==================== =========================================================
"""


import os
from glob import glob
from pydrive.auth import GoogleAuth
from pydrive.drive import GoogleDrive
# from google.colab import auth
from oauth2client.client import GoogleCredentials


__all__ = ['gdrive_downloader',
           'gdrive_uploader']


def gdrive_downloader(gdrive_url_id, pattern='*', download_path='./data'):
    """Download files in Google Drive.

    Download files in Googel Drive to the given path.

    Parameters
    ----------
    gdrive_url_id: str
        An URL ID of an Google Drive directory which contains files to
        download.
        `https://drive.google.com/drive/folders/<google drive URL ID>`.

    pattern: str (default: '*')
        A pattern of regular expression to filter file in the target directory.

    download_path: str (default: './data')
        A target directory to download files in given URL ID.

    Returns
    -------
    None
        Nothing is returned.

    See Also
    --------
    `PyDrive`

    Examples
    --------
    >>> import unipy.util.gdrive import gdrive_downloader
    >>> gdrive_path_id = '1LA5334-SZdizcFqkl4xO8Hty7w1q0e8h'
    >>> up.gdrive_downloader(gdrive_path_id)

    """
    # 1. Authenticate and create the PyDrive client.
    # auth.authenticate_user()
    gauth = GoogleAuth()
    gauth.credentials = GoogleCredentials.get_application_default()
    drive = GoogleDrive(gauth)

    # 2. Create a directory for download.
    try:
        os.makedirs(download_path, exist_ok=False)
        print("Creating a directory: '{path}'".format(path=download_path))
    except FileExistsError:
        print("Directory Exists: '{path}'".format(path=download_path))

    # 3. Get a list of target files.
    file_list = drive.ListFile(
    {'q': "'{url_id}' in parents".format(url_id=gdrive_url_id)}).GetList()

    # 4. Download it.
    for file in file_list:
        # 3. Create & download by id.
        print('title: %s, id: %s' % (file['title'], file['id']))
        fname = os.path.join('data', file['title'])
        print('Downloading {fname} ...'.format(fname=fname))
        f_ = drive.CreateFile({'id': file['id']})
        f_.GetContentFile(fname)

    print('\nDownload Finished.')


def gdrive_uploader(gdrive_url_id, pattern='*', src_dir='./data'):
    """Download files in Google Drive.

    Download files in Googel Drive to the given path.

    Parameters
    ----------
    gdrive_url_id: str
        An URL ID of an Google Drive directory to upload files.
        `https://drive.google.com/drive/folders/<google drive URL ID>`.

    pattern: str (default: '*')
        A pattern of regular expression to filter file in the target directory.

    src_dir: str (default: './data')
        A source directory to upload files in given URL ID.

    Returns
    -------
    None
        Nothing is returned.

    See Also
    --------
    `PyDrive`

    Examples
    --------
    >>> import unipy.util.gdrive import gdrive_uploader
    >>> gdrive_path_id = '1LA5334-SZdizcFqkl4xO8Hty7w1q0e8h'
    >>> up.gdrive_uploader(gdrive_path_id)

    """
    # 1. Authenticate and create the PyDrive client.
    #auth.authenticate_user()
    gauth = GoogleAuth()
    gauth.credentials = GoogleCredentials.get_application_default()
    drive = GoogleDrive(gauth)

    # 2. Get a list of target files.
    print("Uploading: '{path}'".format(path=src_dir))
    file_list = glob(src_dir + '/*')
    gdrive_exist_list = drive.ListFile(
        {'q': "'{url_id}' in parents".format(url_id=gdrive_url_id)}).GetList()
    gdrive_exist_name = [f['title'] for f in gdrive_exist_list]

    for file in file_list:

        fname = file.split('/')[-1]

        if fname in gdrive_exist_name:
            print("'{fname}' File exists. Updating it...".format(fname=fname))
            f_ = [f for f in gdrive_exist_list if f['title'] == fname][0]
        else:
            print("Uploading '{fname}' ...".format(fname=fname))
            f_ = drive.CreateFile({'title': fname,
                                   "parents" : [{"id": gdrive_url_id}]})
        f_.SetContentFile(file)
        f_.Upload()

    print('\nUpload Finished.')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Jun 11 01:42:23 2017

@author: pydemia
"""


import os
import sys
import getpass
import paramiko
import re


"""
Key Stroke Code
http://academic.evergreen.edu/projects/biophysics/technotes/program/ascii_ctrl.htm
"""

# %%

if __name__ == '__main__':

    try:
        kwargsDict = dict(x.split('=', 1) for x in sys.argv[1:])

        host = kwargsDict['-host']
        user = kwargsDict['-user']
        pswd = kwargsDict['-passwd']
        path = kwargsDict['-pythonpath']

    except KeyError:
        host = input('Enter the host address: ')
        user = input('Enter the username: ')
        pswd = getpass.getpass('Enter your password: ')
        path = input('Enter the Python PATH (default: ~/anaconda3/bin): ')

        if path == '':
            path = '~/anaconda3/bin'

    # %%

    with paramiko.SSHClient() as ssh:

        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(host,
                    username=user, password=pswd)

        python_path = path

        dir_command = 'jupyter --runtime-dir'
        exe_command = 'ipython kernel'

        kernel_start_cmd = '/'.join([python_path, exe_command])
        stdin, stdout, stderr = ssh.exec_command(kernel_start_cmd)
        kmsg = stdout.channel.recv(1024)

        comp = re.compile(r'--existing (kernel-\d+.json)')
        kernel_name = comp.findall(str(kmsg))[0]

        get_ipydir_cmd = '/'.join([python_path, dir_command])
        stdin, stdout, stderr = ssh.exec_command(get_ipydir_cmd)
        ipydir = stdout.readlines()[0][:-1]

        json_path = '{kdir}/{kjson}'.format(kdir=ipydir, kjson=kernel_name)
        client_cmd = 'scp -r {user}@{host}:{kpath} ~/'
        cl_cmd = client_cmd.format(user=user, host=host, kpath=json_path)
        os.system(cl_cmd)

        print('json : ~/' + kernel_name)

        while True:
            end_cmd = input("Enter 'exit' to close: ")
            if end_cmd == 'exit':
                channel = ssh.invoke_shell()
                channel.send('\1C')  # 'Ctrl + \' to exit in remote
                stdin, stdout, stderr = ssh.exec_command('rm ' + json_path)
                break

        # while True:
        #     time.sleep(1)



"""Pre-made Dataset Provider.
"""

import pandas as pd
import tarfile
import os
from os.path import dirname, abspath

__all__ = ['init',
           'reset',
           'ls',
           'load']


def init():
    """
    Summary
    `unipy` package has some famous datasets. This function unzip the embedded
    dataset to use.
    Parameters
    ----------

    Returns
    -------
    None

    See Also
    --------
    Examples
    --------
    >>> import unipy.dataset.api as dm
    >>> dm.init()
    ['iris', 'births_small', 'anscombe', 'nutrients', 'car90', 'cars',
     'breast_cancer', 'winequality_red', 'german_credit_scoring_fars2008',
     'winequality_white', 'tips', 'air_quality', 'diabetes', 'births_big',
     'adult', 'titanic']
    """
    filepath = dirname(abspath(__file__))
    filename = filepath + '/resources.tar.gz'
    tar = tarfile.open(filename)
    filelist = list(set(map(lambda x: x.split('/')[0], tar.getnames())))
    tar.extractall(filepath)
    tar.close()
    print(filelist)


def reset():
    """
    Summary
    This function unzip the embedded dataset to use. Equal to `dm.init()`
    Parameters
    ----------

    Returns
    -------
    None

    See Also
    --------
    Examples
    --------
    >>> import unipy.dataset.api as dm
    >>> dm.reset()
    """
    filepath = dirname(abspath(__file__))
    filename = filepath + '/resources.tar.gz'
    tar = tarfile.open(filename)
    tar.extractall(filepath)
    tar.close()


def ls():
    """
    Summary
    This function shows the list of the dataset.
    Parameters
    ----------

    Returns
    -------
    list

    See Also
    --------
    Examples
    --------
    >>> import unipy.dataset.api as dm
    >>> dm.init()
    ['iris', 'births_small', 'anscombe', 'nutrients', 'car90', 'cars',
     'breast_cancer', 'winequality_red', 'german_credit_scoring_fars2008',
     'winequality_white', 'tips', 'air_quality', 'diabetes', 'births_big',
     'adult', 'titanic']
    >>> dm.ls()
    ['iris', 'births_small', 'anscombe', 'nutrients', 'car90', 'cars',
     'breast_cancer', 'winequality_red', 'german_credit_scoring_fars2008',
     'winequality_white', 'tips', 'air_quality', 'diabetes', 'births_big',
     'adult', 'titanic']
    """
    filepath = dirname(abspath(__file__))
    filename = filepath + '/resources.tar.gz'
    tar = tarfile.open(filename)
    filelist = list(set(map(lambda x: x.split('/')[0], tar.getnames())))
    dirclist = os.listdir(filepath)
    datalist = list(filter(lambda x: x in filelist, dirclist))
    datalist.sort()

    print(datalist)
    return(datalist)


def load(pick):
    """
    Summary
    This function returns a dataset you select.
    Parameters
    ----------
    pick: `str` or `int`.
        You can load a dataset by its name or its index from the list of
        `dm.ls()`. Indices start with 0.

    Returns
    -------
    pandas.DataFrame

    See Also
    --------
    Examples
    --------
    >>> import unipy.dataset.api as dm
    >>> dm.init()
    ['iris', 'births_small', 'anscombe', 'nutrients', 'car90', 'cars',
     'breast_cancer', 'winequality_red', 'german_credit_scoring_fars2008',
     'winequality_white', 'tips', 'air_quality', 'diabetes', 'births_big',
     'adult', 'titanic']
    >>> data = dm.load('anscombe')
    Dataset : anscombe
    >>> data = dm.load(2)
    Dataset : anscombe
    """
    filepath = dirname(abspath(__file__))
    dataname = pick

    if type(pick) is str:
        datafile = filepath + '/{dataset}/{dataset}.data'.format(dataset=dataname)
        data = pd.read_csv(open(datafile, 'rb'), sep=",")

    elif type(pick) is int:
        filepath = dirname(abspath(__file__))
        filename = filepath + '/resources.tar.gz'
        tar = tarfile.open(filename)
        filelist = list(set(map(lambda x: x.split('/')[0], tar.getnames())))
        dirclist = os.listdir(filepath)
        datalist = list(filter(lambda x: x in filelist, dirclist))
        datalist.sort()

        dataname = datalist[pick]
        datafile = filepath + '/{dataset}/{dataset}.data'.format(dataset=dataname)
        data = pd.read_csv(open(datafile, 'rb'), sep=",")

    print("Dataset : {}".format(dataname))
    return data



# -*- coding: utf-8 -*-
"""
unipy
=====
Provides
  1. Data Handling Tools
  2. Statistical Functions.
  3. Function Wrappers to profile
  4. Generally-used Plots

How to use
----------
In terms of Data science, Data Preprocessing & Plotting is one of the most
annoying parts of Data Analysis. ``unipy`` offers you many functions maybe
once you have tried to search in ``google`` or ``stackoverflow``.

The docstring examples assume that `unipy` has been imported as `up`::
  >>> import unipy as up

Code snippets are indicated by three greater-than signs::
  >>> x = 42
  >>> x = x + 1

Use the built-in ``help`` function to view a function's docstring::
  >>> help(np.sort)
  ... # doctest: +SKIP

General-purpose documents like a glossary and help on the basic concepts
of numpy are available under the ``docs`` sub-module::
  >>> from unipy import docs
  >>> help(docs)
  ... # doctest: +SKIP

Available subpackages
---------------------
dataset
    Some famous datasets like ``iris``, ``titanic`` and ``adult``
image
    Image transformation tools.
math
    Mathmatical core functions for ``unipy`` itself
plots
    Most used plots
stats
    Statistic tools
tools
    Data handling tools
utils
    High-level wrappers & Python function decorators
unipy_test
    Test-codes of ``unipy``

"""


from unipy.__version__ import __version__

# from unipy import core
# from unipy import dataset
from unipy import math
from unipy import image
from unipy import plots
from unipy import stats
from unipy import tools
from unipy import utils

from unipy.core import *
from unipy.dataset import *
from unipy.math import *
from unipy.plots import *
from unipy.image import *
from unipy.stats import *
from unipy.tools import *
from unipy.utils import *

__all__ = []
__all__ += ['__version__']

# __all__ += core.__all__
__all__ += math.__all__
__all__ += plots.__all__
__all__ += image.__all__
__all__ += stats.__all__

# __all__ += dataset.__all__
__all__ += tools.__all__
__all__ += utils.__all__



"""Sample expensive codes for test.
"""


import time
import random


if __name__ == '__main__':
    def calculate_z_serial_purepython(max_iter, zs, cs):
        output = [0] * len(zs)
        for i in range(len(zs)):
            n = 0
            z = zs[i]
            c = cs[i]
            while abs(z) < 2 and n < max_iter:
                z = z * z + c
                n += 1

            output[i] = n

        return output


    def juliaset_calculator(desired_width=1000, max_iter=300):
        """This Creates zs, cs lists,
        then generate Julia-set and print it.
        """

        x1, x2, y1, y2 = -1.8, -1.8, -1.8, -1.8
        c_real, c_imag = -.62772, -.42193

        x_step = (float(x2 - x1) / float(desired_width))
        y_step = (float(y1 - y2) / float(desired_width))

        x = []
        y = []

        ycoord = y2
        while ycoord > y1:
            y.append(ycoord)
            ycoord += y_step

        xcoord = x1
        while xcoord < x2:
            x.append(xcoord)
            xcoord += x_step

        zs = []
        cs = []
        for ycoord in y:
            for xcoord in x:
                zs.append(complex(xcoord, ycoord))
                cs.append(complex(c_real, c_imag))

        print('Length of x:', len(x))
        print('Total elements:', len(zs))
        start_time = time.time()

        output = calculate_z_serial_purepython(max_iter, zs, cs)
        end_time = time.time()
        elapsed = end_time - start_time
        print(calculate_z_serial_purepython.__name__ + 'took', elapsed, 'seconds')

        assert sum(output) == 33219980


    def montecarlo_pi_estimator(nbr_estimates):
        nbr_trials_in_quarter_unit_circle = 0
        for step in range(int(nbr_estimates)):
            x = random.uniform(0, 1)
            y = random.uniform(0, 1)
            is_in_unit_circle = x ** 2 + y ** 2 <= 1.0
            nbr_trials_in_quarter_unit_circle += is_in_unit_circle

        return nbr_trials_in_quarter_unit_circle


    if __name__ == '__main__':

        juliaset_calculator(desired_width=1000, max_iter=300)

        from multiprocessing import Pool

        nbr_samples_in_total = 1e+8

        pool = Pool(processes=4)
        nbr_samples_per_worker = nbr_samples_in_total / 4

        res = pool.map(montecarlo_pi_estimator, [nbr_samples_per_worker] * 4)
        pi_estimate = sum(res) * 4 / nbr_samples_in_total
        print('Estimated pi:', pi_estimate)

        pass

    # list(filter(lambda col: col not in ['capital_gain'], test.columns))
    # [member for member in test.columns if member not in ['capital_gain']]



# -*- coding: utf-8 -*-
"""
Created on Sun Jan  8 03:46:03 2017

@author: Young Ju Kim
"""


from unipy.stats.formula import *
from unipy.stats.hypo_test import *
from unipy.stats.feature_selection import *
from unipy.stats.metrics import *



#-*- coding: utf-8 -*-
# This file is automatically generated by 'setup.py'
"""
Created on 2017-01-05 20:53:15
Modified on 2018-11-08 18:27:37

@author: Young Ju Kim
"""


__all__ = ['__version__']


__version__ = '0.1.12'




# -*- coding: utf-8 -*-
"""
Created on Thu Jan  5 20:55:26 2017

@author: Young Ju Kim
"""


if __name__ == '__main__':
    #%% Sample datasets
    import unipy.dataset.api as dm

    # Extract Datasets for the first time
    dm.init()

    # Reset Datasets
    dm.reset()

    # Get a Dataset list
    dm.ls()

    # Load Datasets
    wine1 = dm.load('winequality_red')
    wine2 = dm.load('winequality_white')

    #%% VIF
    #from unipy.stats import feature_selection_vif as fsv

    #res, drp = fsv(wine1, thresh=5.0)


    #%% Inter-Action
    #from unipy.stats import calculate_interaction
    #rankedInter, regCoef, interActTbl = calculate_interaction(ranked, pvTbl, targetedCol, ranknum=10)



"""Docstring for ``decorator``.

========================
Function Decorator
========================
==================== =========================================================
Profiler
==============================================================================
time_profiler        Function running time command-line profiler.
time_logger          Function running time log profiler.
==================== =========================================================
==================== =========================================================
Commandline printout
==============================================================================
job_wrapper          Command-line line dragging tool.
==================== =========================================================
==================== =========================================================
Code Translation
==============================================================================
Infix                Function to operator translator.
infix                Functional API for ``Infix``.
==================== =========================================================
"""


import logging
from datetime import datetime as dt
from functools import partial
from functools import wraps


__all__ = ['time_profiler',
           'time_logger',
           'job_wrapper',
           'Infix',
           'infix']


def time_profiler(func):
    """Print wrapper for time profiling.

    This wrapper prints out start, end and elapsed time.

    Parameters
    ----------
    func: Function
        A function to profile.

    Returns
    -------
    Function
        A wrapped function.

    See Also
    --------
    ``functools.wraps``
    ``decorator``

    Examples
    --------
    >>> import unipy as up
    >>> @up.time_profiler
    ... def afunc(i):
    ...     return len(list(range(i)))
    ...
    >>> res = afunc(58)
    (afunc) Start   : 2018-06-20 22:11:35.511374
    (afunc) End     : 2018-06-20 22:11:35.511424
    (afunc) Elapsed :             0:00:00.000050
    >>> res
    58

    """
    @wraps(func)
    def profiler(*args, **kwargs):

        start_tm = dt.now()
        print("(%s) Start   : %26s" % (func.__name__, start_tm))

        res = func(*args, **kwargs)
        end_tm = dt.now()
        print("(%s) End     : %26s" % (func.__name__, end_tm))

        elapsed_tm = end_tm - start_tm
        print("(%s) Elapsed : %26s" % (func.__name__, elapsed_tm))
        return res

    return profiler


def time_logger(func):
    """Logging wrapper for time profiling.

    This wrapper logs start, end and elapsed time.

    Parameters
    ----------
    func: Function
        A function to profile.

    Returns
    -------
    Function
        A wrapped function.

    See Also
    --------
    ``functools.wraps``
    ``decorator``

    Examples
    --------
    >>> import unipy as up
    >>> @up.time_logger
    ... def afunc(i):
    ...     return len(list(range(i)))
    ...
    >>> res = afunc(58)
    (afunc) Start   : 2018-06-20 22:11:35.511374
    (afunc) End     : 2018-06-20 22:11:35.511424
    (afunc) Elapsed :             0:00:00.000050
    >>> res
    58

    """
    @wraps(func)
    def logger(*args, **kwargs):

        start_tm = dt.now()
        logging.info("(%s) Start   : %26s" % func.__name__ + str(start_tm))

        res = func(*args, **kwargs)

        end_tm = dt.now()
        logging.info("(%s) End     : %26s" % func.__name__ + str(end_tm))

        elapsed_tm = end_tm - start_tm
        logging.info("(%s) Elapsed : %26s" % func.__name__ + str(elapsed_tm))

        return res

    return logger


def job_wrapper(func):
    """Print wrapper for time profiling.

    This wrapper prints out start & end line.

    Parameters
    ----------
    func: Function
        A function to separate print-line job.

    Returns
    -------
    Function
        A wrapped function.

    See Also
    --------
    ``functools.wraps``
    ``decorator``

    Examples
    --------
    >>> import unipy as up
    >>> @up.job_wrapper
    ... def afunc(i):
    ...     return len(list(range(i)))
    ...
    >>> afunc(458)
    ----------- [afunc] START -----------

    -----------  [afunc] END  -----------

    afunc : 0:00:00.000023

    458

    """
    @wraps(func)
    def wrapper(*args, **kwargs):

        # Message Length = 40 + 2 (\n)
        dash_len = int((40 - len(func.__name__) - 12) / 2)

        start_msg = '-'*dash_len + ' [{}] START '.format(func.__name__) + \
                    '-'*dash_len + '\n'
        print(start_msg)
        start_tm = dt.now()

        res = func(*args, **kwargs)

        end_tm = dt.now()
        end_msg = '-'*dash_len + '  [{}] END  '.format(func.__name__) + \
                  '-'*dash_len + '\n'
        print(end_msg)
        print("{} :".format(func.__name__), end_tm - start_tm, '\n')

        return res

    return wrapper


class Infix(object):
    """Wrapper for define an operator.

    This wrapper translates a function to an operator.

    Returns
    -------
    Function
        A wrapped function.

    See Also
    --------
    ``functools.partial``
    ``decorator``

    Examples
    --------
    >>> @Infix
    ... def add(x, y):
    ...     return x + y
    ...
    >>> 5 |add| 6
    11
    >>> instanceof = Infix(isinstance)
    >>> 5 |instanceof| int
    True

    """
    def __init__(self, func):
        """__init__ method.

        Parameters
        ----------
        function: Function
            A function to translate as an operator.
        """
        self.func = func

    def __or__(self, other):
        return self.func(other)

    def __ror__(self, other):
        return Infix(partial(self.func, other))

    def __rshift__(self, other):
        return self.func(other)

    def __rlshift__(self, other):
        return Infix(partial(self.func, other))

    def __call__(self, lhs, rhs):
        return self.func(lhs, rhs)


def infix(func):
    """A functional API for Infix decorator.

    Returns
    -------
    Function
        A wrapped function.

    See Also
    --------
    ``unipy.utils.wrapper.infix``

    Examples
    --------
    >>> @infix
    ... def add(x, y):
    ...     return x + y
    ...
    >>> 5 |add| 6
    11
    >>> instanceof = infix(isinstance)
    >>> 5 |instanceof| int
    True

    """
    return Infix(func)



# -*- coding: utf-8 -*-
"""
Created on Thu Jan  5 20:55:26 2017

@author: Young Ju Kim
"""



if __name__ == '__main__':
    # Import dataset manager
    import unipy.dataset.api as dm

    # Extract Datasets for the first time
    dm.init()

    # Reset Datasets
    dm.reset()

    # Get a Dataset list
    dm.ls()

    # Load Datasets
    data = dm.load('anscombe')
    data = dm.load(2)



"""Statistical Hypothesis Tests.

"""


import warnings
import numpy as np
import pandas as pd
import scipy.stats as st
import statsmodels.stats.api as smt
import statsmodels.formula.api as smf


__all__ = ['f_test',
           'f_test_formula',
           'anova_test',
           'anova_test_formula',
           'chisq_test',
           'fisher_test']


def f_test(a, b, scale=1, alternative='two-sided',
           conf_level=.95, *args, **kwargs):
    """F-Test.

    """
    assert 0 < scale <= 1
    assert 0 < conf_level <= 1

    dfn = len(a) - 1
    dfd = len(b) - 1

    if (len(a) or len(b)) < 2:

        print('Observations are insufficient.')
        f_statistics = np.nan
        p_value = np.nan
        conf_min, conf_max = np.nan, np.nan

    else:

        try:
            f_statistics = np.array(a).var() / np.array(b).var()
        except ZeroDivisionError:
            f_statistics = np.nan

        p_value = st.f.cdf(np.abs(f_statistics), dfn, dfd, scale=scale)
        # p_value = distributions.t.sf(np.abs(t), df)

        conf_interval = st.f.interval(conf_level, dfn, dfd, scale=scale)
        conf_min, conf_max = np.multiply(f_statistics, conf_interval)

        if alternative == 'two-sided':
            p_value = 2 * np.min([p_value, 1-p_value])

        elif alternative == 'greater':
            p_value = st.f.sf(f_statistics, dfn, dfd, scale=scale)
            conf_min, conf_max = conf_min, np.inf

        elif alternative == 'less':
            conf_min, conf_max = 0, conf_max

    # print('F-Statistics: %.4g' % f_statistics)
    # print('dfn, dfd: %d %d' % (dfn, dfd) )
    # print('Hypothesized Scale: %d' % scale)
    # print('Confidence Level: %.3g' % conf_level)
    # print('Confidence Interval: %.4g, %.4g' % (conf_min, conf_max))
    # print('P-value: %.4g' % p_value)

    res = pd.Series({'f_statistics': f_statistics,
                     'dfn': dfn,
                     'dfd': dfd,
                     'hypo_scale': scale,
                     'conf_level': conf_level,
                     'conf_interval': (conf_min, conf_max),
                     'p_value': p_value})

    return res


def f_test_formula(a, b, scale=1, alternative='two-sided',
                   conf_level=.95, *args, **kwargs):
    """F-Test by formula.

    """
    assert 0 < scale <= 1
    assert 0 < conf_level <= 1

    dfn = len(a) - 1
    dfd = len(b) - 1

    if (len(a) or len(b)) < 2:

        print('Observations are insufficient.')
        f_statistics = np.nan
        p_value = np.nan
        conf_min, conf_max = np.nan, np.nan

    else:

        try:
            f_statistics = np.array(a).var() / np.array(b).var()
        except ZeroDivisionError:
            f_statistics = np.nan

        p_value = st.f.cdf(np.abs(f_statistics), dfn, dfd, scale=scale)
        # p_value = distributions.t.sf(np.abs(t), df)

        conf_interval = st.f.interval(conf_level, dfn, dfd, scale=scale)
        conf_min, conf_max = np.multiply(f_statistics, conf_interval)

        if alternative == 'two-sided':
            p_value = 2 * np.min([p_value, 1-p_value])

        elif alternative == 'greater':
            p_value = st.f.sf(f_statistics, dfn, dfd, scale=scale)
            conf_min, conf_max = conf_min, np.inf

        elif alternative == 'less':
            conf_min, conf_max = 0, conf_max


    # print('F-Statistics: %.4g' % f_statistics)
    # print('dfn, dfd: %d %d' % (dfn, dfd) )
    # print('Hypothesized Scale: %d' % scale)
    # print('Confidence Level: %.3g' % conf_level)
    # print('Confidence Interval: %.4g, %.4g' % (conf_min, conf_max))
    # print('P-value: %.4g' % p_value)

    res = pd.Series({'f_statistics': f_statistics,
                     'dfn': dfn,
                     'dfd': dfd,
                     'hypo_scale': scale,
                     'conf_level': conf_level,
                     'conf_interval': (conf_min, conf_max),
                     'p_value': p_value})

    return res


def anova_test(formula, data=None, typ=1):
    """ANOVA Test.

    """
    lin_model = smf.ols(formula, data=data).fit()
    res_anova = smt.anova_lm(lin_model, typ=typ)

    return res_anova, lin_model


def anova_test_formula(formula, data=None, typ=1):
    """ANOVA Test by formula.

    """
    lin_model = smf.ols(formula, data=data).fit()
    res_anova = smt.anova_lm(lin_model, typ=typ)

    return res_anova, lin_model


def chisq_test(data, x=None, y=None, correction=None,
               lambda_=None, margin=True, print_ok=True):
    """Chi-square Test.

    ``lambda_`` gives the power in the Cressie-Read power divergence statistic.
    The default is 1.
    For convenience, lambda_ may be assigned one of the following strings,
    in which case the corresponding numerical value is used:

    Parameters
    ----------
    data: pandas.DataFrame

    x: str (default: None)

    y: str (default: None)

    correction: (default: None)

    lambda_: lambda (default: None)

    margin: Boolean (default: True)

    print_ok: Boolean (default: True)

    """
    dataChi = data[[x, y]].dropna()
    crossT = pd.crosstab(dataChi[x], dataChi[y], margins=margin)

    if margin:
        crossT_raw = crossT.iloc[:-1, :-1]
        (chiSqr,
         pValue,
         freeDeg,
         expectedV_raw) = st.chi2_contingency(crossT_raw,
                                              correction=correction,
                                              lambda_=lambda_)
        expectedV_raw = pd.DataFrame(expectedV_raw,
                                     index=crossT_raw.index,
                                     columns=crossT_raw.columns)

        expectedV = expectedV_raw.copy()
        expectedV['All'] = expectedV.sum(axis=1)
        expectedV = expectedV.append(expectedV.sum(axis=0).rename('All'))

    else:
        (chiSqr,
         pValue,
         freeDeg,
         expectedV_raw) = st.chi2_contingency(crossT,
                                              correction=correction,
                                              lambda_=lambda_)
        expectedV_raw = pd.DataFrame(expectedV_raw,
                                     index=crossT.index,
                                     columns=crossT.columns)

        expectedV = expectedV_raw

    check_chisq_rule = (expectedV_raw[expectedV_raw <= 5].count().sum() <=
                        (expectedV_raw.shape[0] * expectedV_raw.shape[1]) * .2)

    if print_ok:
        print('#'*20 + ' Chi-Square Test ' + '#'*20 + '\n')
        print('x : ' + x)
        print('y : ' + y)
        print('\n## Contingency Table')
        print('-'*50)
        print(crossT)
        print('-'*50)

        msg = """
## Chi-square Statistic : {}
## P-value : {}
## Degrees of Freedom : {}
"""

        print(msg.format(chiSqr, pValue, freeDeg))
        print('\n' + '## Expected Values')
        print('-'*60)
        print(expectedV)
        print('-'*60 + '\n')

    if not check_chisq_rule:
        w_msg1 = 'Warning: The precondition of Chi-Square was not implemented.'
        w_msg2 = 'Chi-squared approximation may be incorrect.'
        w_msg = '\n'.join([w_msg1, w_msg2])
        warnings.warn(w_msg)

    return pd.Series({'cross_t': crossT,
                      'chisq-stat': chiSqr,
                      'p_value': pValue,
                      'df': freeDeg,
                      'expected': expectedV})


def fisher_test(data, x=None, y=None, alternative='two-sided',
                margin=True, print_ok=True):
    """Fisher's Exact Test.

    """
    dataChi = data[[x, y]].dropna()
    crossT = pd.crosstab(dataChi[x], dataChi[y], margins=margin)

    if margin:
        crossT_raw = crossT.iloc[:-1, :-1]
        odds_ratio, p_value = st.fisher_exact(crossT_raw,
                                              alternative=alternative)
    else:
        odds_ratio, p_value = st.fisher_exact(crossT,
                                              alternative=alternative)

    if print_ok:
        print('#'*20 + ' Fisher\'s Exact Test ' + '#'*20 + '\n')
        print('x : ' + x)
        print('y : ' + y)
        print('\n## Contingency Table')
        print('-'*50)
        print(crossT)
        print('-'*50)

        msg = """
## Odds Ratio : {}
## P-value : {}
## Alternative Hypothesis : {}
"""

        print(msg.format(odds_ratio, p_value, alternative))
        print('-'*60 + '\n')

    return pd.Series({'odds_ratio': odds_ratio, 'p_value': p_value})


if __name__ == '__main__':

    a = [65,75,83,75,89,95,80,69,76,79,81,90,75,72,83,92,61,87,62.80]
    b = [54,65,91,76,79,78,84,97,88,75,72,68,76,85,72,88]

    # %%

    f_test(a, b)
    f_test(a, b, alternative='greater')
    f_test(a, b, alternative='less')



# -*- coding: utf-8 -*-
"""
Created on Tue Aug  8 01:04:13 2017

@author: Young Ju Kim
"""


__all__ = ['from_formula']


def from_formula(formula):
    """R-style Formula Formatting.

    """
    yCol = formula.replace(' ', '').split('~')[0].strip()
    xCol = formula.replace(' ', '').split('~')[1].strip().split('+')

    return xCol, yCol



"""Complexed Plotting Toolkit.

"""


import numpy as np
import pandas as pd
import collections
import matplotlib.pyplot as plt


__all__ = ['point_boxplot',
           'point_boxplot_axis',
           'mosaic_plot']


def point_boxplot(data, groupby=None, value=None,
                  rot=90, spread=.2,
                  dot_size=15., dot_color='b', dot_alpha=.2,
                  figsize=(12, 9), *args, **kwargs):
    """Boxplot with points.

    Draw boxplots by given keys(groupby, value).

    Parameters
    ----------
    data: pandas.DataFrame
        a dataset.

    groupby: str or list-like (default: None)
        A key column to separate. (X-axis, categorical)
        When ``str``, it should be a column name to groupby.
        When ``list-like``, it contains a column name to groupby.

    value: str or list-like (default: None)
        A key column to get values. (Y-axis, numerical)
        When ``str``, it should be a column name of values.
        When ``list-like``, it contains a column name of values.

    rot: int (default: 90)
        A rotation angle to show X-axis labels.

    spread: float (default: .2)
        A spread ratio of points.
        The bigger, the pointing distribution width are broader.

    dot_size: float (default: 15.)
        A size of each points.

    dot_color: int (default: 'b')
        A color name of each points.

    dot_alpha: float (default: .2)
        A transparency value of each points.

    Returns
    -------
    matplotlib.figure.Figure
        A plot figure.

    Exceptions
    ----------
    AssertionError
        It is raised when two or more names are given to
        ``groupby`` or ``value``.

    See also
    --------
    ``pandas.DataFrame.boxplot``
    ``matplotlib.pyplot``

    Examples
    --------
    >>> import unipy.dataset.api as dm
    >>> from unipy.plots import point_boxplot
    >>> dm.init()
    >>> data = dm.load('iris')
    Dataset : iris
    >>> tmp = point_boxplot(data, groupby='species', value='sepal_length')

    """
    if isinstance(groupby, (list, tuple)):
        groupby_list = groupby
    elif isinstance(groupby, str):
        groupby_list = [groupby]

    if isinstance(value, (list, tuple)):
        value_list = value
    elif isinstance(value, str):
        value_list = [value]

    assert len(groupby_list) == 1, "'groupby': should be a single column"
    assert len(value_list) == 1, "'value': should be a single column"

    flierprops = dict(marker='o', markerfacecolor='white',
                      alpha=1., markersize=5,
                      linestyle='none', markeredgewidth=.7)

    grouped = data.groupby(groupby_list)[value_list]

    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=figsize)

    data.boxplot(by=groupby_list, column=value_list, rot=rot, ax=ax,
                 flierprops=flierprops, showfliers=True,
                 showmeans=True)

    for i, (key, grp) in enumerate(grouped):
        axis_y_val = grp[value]
        axis_x_loc = np.random.normal(i + 1,
                                      spread / len(grouped),
                                      len(axis_y_val))
        ax.scatter(x=axis_x_loc, y=axis_y_val,
                   s=dot_size, c=dot_color, alpha=dot_alpha)

    fig.tight_layout()

    return fig


def point_boxplot_axis(data, groupby=None, value=None,
                       rot=90, spread=.2,
                       dot_size=15., dot_color='b', dot_alpha=.2,
                       share_yrange=True,
                       figsize=(12, 9),
                       *args, **kwargs):
    """Boxplot with points, horizontally seperated.

    Draw boxplots by given keys(groupby, value).

    Parameters
    ----------
    data: pandas.DataFrame
        a dataset.

    groupby: str or list-like (default: None)
        A key column to separate. (X-axis, categorical)
        When ``str``, it should be a column name to groupby.
        When ``list-like``, it contains a column name to groupby.

    value: str or list-like (default: None)
        A key column to get values. (Y-axis, numerical)
        When ``str``, it should be a column name of values.
        When ``list-like``, it contains a column name of values.

    rot: int (default: 90)
        A rotation angle to show X-axis labels.

    spread: float (default: .2)
        A spread ratio of points.
        The bigger, the pointing distribution width are broader.

    dot_size: float (default: 15.)
        A size of each points.

    dot_color: int (default: 'b')
        A color name of each points.

    dot_alpha: float (default: .2)
        A transparency value of each points.

    share_yrange: Boolean (defalut: True)
        False then each Y-axis limit of boxplots will draw independent.

    Returns
    -------
    matplotlib.figure.Figure
        A plot figure.

    Exceptions
    ----------
    AssertionError
        It is raised when two or more names are given to
        ``groupby`` or ``value``.

    See also
    --------
    ``pandas.DataFrame.boxplot``
    ``matplotlib.pyplot``

    Examples
    --------
    >>> import unipy.dataset.api as dm
    >>> from unipy.plots import point_boxplot_axis
    >>> dm.init()
    >>> data = dm.load('iris')
    Dataset : iris
    >>> tmp = point_boxplot_axis(data,
    ...                          groupby='species',
    ...                          value='sepal_length',
    ...                          share_yrange=True)

    """
    if isinstance(groupby, (list, tuple)):
        groupby_list = groupby
    elif isinstance(groupby, str):
        groupby_list = [groupby]

    if isinstance(value, (list, tuple)):
        value_list = value
    elif isinstance(value, str):
        value_list = [value]

    assert len(groupby_list) == 1, "'groupby': should be a single column"
    assert len(value_list) == 1, "'value': should be a single column"

    flierprops = dict(marker='o', markerfacecolor='white',
                      alpha=1., markersize=5,
                      linestyle='none', markeredgewidth=.7)

    grouped = data.groupby(groupby_list)[value_list]
    ylim_min = data[value_list].min()[0]
    ylim_max = data[value_list].max()[0]

    fig, axes = plt.subplots(nrows=1, ncols=len(grouped), figsize=figsize)

    for ax, (i, (key, subdata)) in zip(axes.flatten(), enumerate(grouped)):
        subdata.boxplot(column=value_list, rot=rot, ax=ax,
                        flierprops=flierprops, showfliers=True,
                        showmeans=True)

        axis_y_val = subdata[value_list]
        axis_x_loc = np.random.normal(1,
                                      spread / len(grouped),
                                      len(axis_y_val))
        ax.scatter(x=axis_x_loc, y=axis_y_val,
                   s=dot_size, c=dot_color, alpha=dot_alpha)
        if share_yrange:
            ax.set_ylim(ylim_min, ylim_max)

    fig.tight_layout()

    return fig


def mosaic_plot(
    data, groupby=None, col_list=None, show_values=True,
    rot=90, width=.9,
    figsize=(12, 9), *args, **kwargs,
    ):
    """Mosaic Plot via Stacked bar plots.

    Draw plots by given keys(groupby, value).

    Parameters
    ----------
    data: pandas.DataFrame
        a dataset.

    groupby: str or list-like (default: None)
        A key column to separate. (X-axis, categorical)
        When ``str``, it should be a column name to groupby.
        When ``list-like``, it contains a column name to groupby.

    col_list: str or list-like (default: None)
        A key column to get values. (Y-axis, numerical)
        When ``str``, it should be column names of values.
        When ``list-like``, it contains column names of values.

    rot: int (default: 90)
        A rotation angle to show X-axis labels.

    show_values: boolean (default: True)
        Choose If `n` is annotated.

    Returns
    -------
    matplotlib.figure.Figure
        A plot figure.

    Exceptions
    ----------
    AssertionError
        It is raised when two or more names are given to
        ``groupby`` or ``value``.

    See also
    --------
    ``pandas.DataFrame.barplot``
    ``matplotlib.pyplot``

    Examples
    --------
    >>> import unipy.dataset.api as dm
    >>> from unipy.plots import mosaic_plot
    >>> dm.init()
    >>> data = dm.load('adult')
    Dataset : iris
    >>> tmp = mosaic_plot(data, groupby='native_country',
    ... col_list=['workclass', 'education'])

    """


    if isinstance(groupby, (list, tuple)):
        groupby_list = groupby
    elif isinstance(groupby, str):
        groupby_list = [groupby]

    if isinstance(col_list, (list, tuple)):
        col_list = col_list
    elif isinstance(col_list, str):
        col_list = [col_list]

    assert len(groupby_list) == 1, "'groupby': should be a single column"

    grouped = data[groupby_list + col_list].groupby(groupby_list)

    fig, ax = plt.subplots(
        nrows=len(grouped), ncols=1, figsize=figsize,
    )
    for ax, (key, grp) in zip(ax, grouped):
        tmp_table = grp[col_list]
        freq_table = tmp_table.apply(pd.value_counts)
        #freq_ratio_table = freq_table.apply(lambda x: x / np.nansum(x))
        freq_table.T.plot(
            kind='bar', stacked=True,
            #figsize=(1.5 * tmp_table.shape[1], 12),
            ax=ax,
            rot=rot, width=width, edgecolor='white',
        )
        ax.legend(
            title=groupby_list[0],
            loc='center right',
            bbox_to_anchor=(1.11, .5),
        )

        if show_values:
            for p in ax.patches:
                width, height = p.get_width(), p.get_height()
                x, y = p.get_xy()
                ax.annotate(
                    f'{height:.0f}',
                    (
                        p.get_x() + .5 * width,
                        p.get_y() + .5 * height,
                    ),
                    ha='center', va='bottom',
                )

    return fig


def sector_plot():
    pass



"""Feature selection.

"""


# import numba as nb
import numpy as np
import pandas as pd
import sklearn as skl
import matplotlib.pyplot as plt
from sklearn.linear_model import Lasso
from unipy.stats.formula import from_formula


__all__ = ['lasso_rank',
           'feature_selection_vif']


# Defining a Lasso generic function
def _lasso_for_loop(data, X=None, y=None, alpha=.0001, *args, **kwargs):

    # Fit to the model
    lassoReg = Lasso(alpha=alpha, fit_intercept=True,
                     normalize=True, precompute=False,
                     max_iter=1e5, tol=1e-7,
                     warm_start=False, positive=False,
                     selection='cyclic', *args, **kwargs)

    lassoReg.fit(data[X], data[y].squeeze())
    yPredict = lassoReg.predict(data[X])

    # Return the result in pre-defined format
    rss = np.sum((yPredict - data[y].squeeze()) ** 2)
    ret = [rss]
    ret.extend([lassoReg.intercept_])
    ret.extend(lassoReg.coef_)

    return ret, yPredict


def lasso_rank(formula=None, X=None, y=None, data=None,
               alpha=np.arange(1e-5, 1e-2, 1e-4), k=2, plot=False,
               *args, **kwargs):
    """Feature selection by LASSO regression.

    Parameters
    ----------
    formula:
        R-style formula string

    X: list-like
        Column values for X.

    y: list-like
        A column value for y.

    data: pandas.DataFrame
        A DataFrame.

    alpha: Iterable
        An Iterable contains alpha values.
    k: int
        Threshold of coefficient matrix

    plot: Boolean (default: False)
        True if want to plot the result.

    Returns
    -------
    rankTbl: pandas.DataFrame
        Feature ranking by given ``k``.

    minIntercept: pandas.DataFrame
        The minimum intercept row in coefficient matrix.

    coefMatrix: pandas.DataFrame
        A coefficient matrix.

    kBest: pandas.DataFrame
        When Given ``k``, The best intercept row in coefficient matrix.

    kBestPredY: dict
        A predicted ``Y`` with ``kBest`` alpha.

    Example
    -------
    >>> import unipy.dataset.api as dm
    >>> dm.init()
    ['cars', 'anscombe', 'iris', 'nutrients', 'german_credit_scoring_fars2008', 'winequality_red', 'winequality_white', 'titanic', 'car90', 'diabetes', 'adult', 'tips', 'births_big', 'breast_cancer', 'air_quality', 'births_small']
    >>> wine_red = dm.load('winequality_red')
    Dataset : winequality_red
    >>>
    >>> ranked, best_by_intercept, coefTbl, kBest, kBestPred = lasso_rank(X=wine_red.columns.drop('quality'), y=['quality'], data=wine_red)
    >>> ranked
                      rank  lasso_coef  abs_coef
    volatile_acidity     1   -0.675725  0.675725
    alcohol              2    0.194865  0.194865
    >>> best_by_intercept
                          RSS  Intercept  fixed_acidity  volatile_acidity  \
    alpha_0.00121  691.956364   3.134874       0.002374         -1.023793

                   citric_acid  residual_sugar  chlorides  free_sulfur_dioxide  \
    alpha_0.00121          0.0             0.0  -0.272912                 -0.0

                   total_sulfur_dioxide  density   pH  sulphates   alcohol  \
    alpha_0.00121             -0.000963     -0.0 -0.0   0.505956  0.264552

                   var_count
    alpha_0.00121          6
    >>>
    """
    if formula is not None:
        X, y = from_formula(formula)
    else:
        X = list(X)
        y = y

    # Iterate over the alpha values
    coefMatrix = {'alpha_%.5f' % a: _lasso_for_loop(data, X=X, y=y, alpha=a, *args, **kwargs)[0] for a in alpha}
    predict    = {'alpha_%.5f' % a: _lasso_for_loop(data, X=X, y=y, alpha=a, *args, **kwargs)[1] for a in alpha}

    coefMatrix = pd.DataFrame(coefMatrix).T
    coefMatrix.columns = ['RSS', 'Intercept'] + X
    coefMatrix['var_count'] = coefMatrix.apply(np.count_nonzero, axis=1) - 2

    # Filter by thresh >= var_count
    kBest = coefMatrix[coefMatrix['var_count'] <= k]
    kBest = kBest.loc[kBest[['var_count']].idxmax()]
    kBest = kBest.loc[kBest[['Intercept']].idxmin()]

    # Minumum Intercept
    minIntercept = coefMatrix.loc[coefMatrix[['Intercept']].idxmin()]

    # Get Predicted Y value
    alphaVal = kBest.index[0]
    kBestPredY = {alphaVal: predict[alphaVal]}

    # Get a Rank Table
    lassoVal = kBest.iloc[:, kBest.squeeze().nonzero()[0].tolist()[2:-1]]
    filteredTbl = pd.concat([lassoVal.T, abs(lassoVal).T], axis=1)
    filteredTbl.columns = ['lasso_coef', 'abs_coef']
    filteredTbl = filteredTbl.sort_values(by='abs_coef', ascending=False)
    filteredTbl['rank'] = range(1, len(filteredTbl) + 1)
    rankTbl = filteredTbl[['rank', 'lasso_coef', 'abs_coef']]

    # Plots
    #fig = plt.figure(figsize=(12, 9))
    #title = 'Top {} variables : absolute coefficient by Lasso'.format(len(filteredTbl))
    #rankTbl['abs_coef'].plot(kind='barh')
    #fig.suptitle(title, fontsize=14, fontweight='bold')
    #plt.tight_layout(pad=5)

    return rankTbl, minIntercept, coefMatrix, kBest, kBestPredY


def feature_selection_vif(data, thresh=5.0):
    '''Stepwise Feature Selection for multivariate analysis.

    It calculates OLS regressions and the variance inflation factors iterating
    all explanatory variables. If the maximum VIF of a variable is over the
    given threshold, It will be dropped. This process is repeated until all
    VIFs are lower than the given threshold.

    Recommended threshold is lower than 5, because if VIF is greater than 5,
    then the explanatory variable selected is highly collinear with the other
    explanatory variables, and the parameter estimates will have large standard
    errors because of this.

    Parameters
    ----------
    data : DataFrame, (rows: observed values, columns: multivariate variables)
        design dataframe with all explanatory variables, as for example used in
        regression

    thresh : int, float
        A threshold of VIF

    Returns
    -------
    Filtered_data : DataFrame
        A subset of the input DataFame

    dropped_List : DataFrame
        'var' column : dropped variable names from input data columns
        'vif' column : variance inflation factor of dropped variables

    Notes
    -----
    This function does not save the auxiliary regression.

    See Also
    --------
    statsmodels.stats.outliers_influence.variance_inflation_factor

    References
    ----------
    http://en.wikipedia.org/wiki/Variance_inflation_factor

    '''
    assert isinstance(data, pd.DataFrame)

    # Create Dropped variable list
    dropped = pd.DataFrame(columns=['var', 'vif'])

    # Startswith 'drop = True'(Assume that some variables will be dropped)
    dropCondition = True

    # Calculate a VIF & Drop columns(variables)
    while dropCondition:

        # 1. Calculate a VIF
        vifDict = {col: vif(data.loc[:, col], data.loc[:, data.columns != col])
                   for col in data.columns}

        # Get the MAXIMUM VIF
        maxVar = max(vifDict, key=vifDict.get)
        maxVal = vifDict[maxVar]

        # 2. IF VIF values are over the threshold, THEN drop it
        if maxVal >= thresh:

            # Keep it
            dropped = dropped.append({'var': maxVar, 'vif': maxVal},
                                     ignore_index=True)

            # Drop it
            data = data.drop(maxVar, axis=1)

            # Print it
            print("Dropping '" + str(maxVar) + "' " + " VIF: " + str(maxVal))

            # Since a variable has been dropped, the assumption remains
            dropCondition = True

        else:

            # No variable dropped, the assumption has been rejected
            dropCondition = False

    # Print Massages
    remainsMsg = '# Remaining Variables '
    msgWrapper = '-' * (len(remainsMsg)+1)

    print('\n' + msgWrapper + '\n' + remainsMsg + '\n' + msgWrapper)
    print(list(data.columns))
    print('\n')

    droppedMsg = '# Dropped Variables '
    msgWrapper = '-' * (len(remainsMsg)+1)
    print('\n' + msgWrapper + '\n' + droppedMsg + '\n' + msgWrapper)
    print(list(dropped.loc[:, 'var']))
    print('\n')

    return data, dropped



"""Utility Objects.

This module provides a number of functions and objects for utility.

hypo_test
---------
- `f_test` -- F-Test.
- `f_test_formula` -- F-Test by formula.
- `anova_test` -- ANOVA Test.
- `anova_test_formula` -- ANOVA Test by formula.
- `chisq_test` -- Chi-square Test.
- `fisher_test` -- Fisher's Exact Test.

feature_selection
-----------------
- `lasso_rank` -- Feature selection by LASSO regression.
- `feature_selection_vif` -- VIF based stepwise feature selection
                             for multivariate analysis.

metrics
-------
- `deviation` -- Deviation.
- `vif` -- Variance inflation factor.
- `mean_absolute_percentage_error` -- Mean Absolute Percentage Error.
- `average_absolute_deviation` -- Average Absolute Deviation.
- `median_absolute_deviation` -- Median Absolute Deviation.
- `calculate_interaction` -- Feature interaction calculation.

formula
-------
- `from_formula` -- R-style Formula Formatting.

"""


from unipy.stats import metrics
from unipy.stats import hypo_test
from unipy.stats import feature_selection
from unipy.stats import formula


from unipy.stats.metrics import *
from unipy.stats.hypo_test import *
from unipy.stats.feature_selection import *
from unipy.stats.formula import *

__all__ = []
__all__ += metrics.__all__
__all__ += hypo_test.__all__
__all__ += feature_selection.__all__
__all__ += formula.__all__



"""Plotting Toolkit.

Boxplot
-------
- `point_boxplot` -- Boxplot with points.
- `point_boxplot_grid` -- Grid-framed boxplot with points.

"""


from unipy.plots import boxplot
from unipy.plots.boxplot import *

__all__ = []
__all__ += boxplot.__all__



"""Image processing toolkit.

houghmatrix
-----------
- `rgb2gras` -- RGB to Grayscale.
- `hough_transform` -- Hough Transformation.
 
"""


from unipy.image import houghmatrix
from unipy.image.houghmatrix import *


__all__ = []
__all__ += houghmatrix.__all__




"""Datasets.

This module offers you well-known datasets.

api
---
- `init` -- Unzip datasets.
- `reset` -- Re-unzip datasets.
- `ls` -- List-up datasets.
- `load` -- Load a dataset.

"""


from unipy.dataset import api

from unipy.dataset.api import *

__all__ = []
__all__ += api.__all__



"""Image Transformation.

"""


import numpy as np
import scipy.ndimage as sni
import matplotlib.image as mim
import matplotlib.pyplot as plt


__all__ = ['rgb2gras',
           'hough_transform']


def rgb2gras(img_array):
  assert(img_array.shape[2] == 3)
  img_gray_array = mim.zeros((img_array.shape[0], img_array.shape[1]), dtype=np.float32)
  for _, __ in range(img_array.shape[0]), range(img_array.shape[1]):
    img_gray_array[_][__] = 0.2989*img_array[_][__][0] + \
      0.5870*img_array[_][__][1] + 0.1140*img_array[_][__][2]
  return img_gray_array


def hough_transform(img_bin, theta_res=1, rho_res=1):
  nR, nC = img_bin.shape
  theta = np.linspace(-90., 0., np.ceil(90. / theta_res) + 1.)
  theta = np.concatenate((theta, -theta[len(theta)-2::-1]))
  
  D = np.sqrt((nR -1)**2 + (nC -1)**2)
  q = np.ceil(D/rho_res)
  nrho = 2*q + 1
  rho = np.linspace(-q*rho_res, q*rho_res, nrho)
  H = np.zeros((n(rho), len(theta)))
  for rowIdx in range(nR):
    for colIdx in range(nC):
      if img_bin[rowIdx, colIdx]:
        for thIdx in range(len(theta)):
          rhoVal = colIdx*np.cos(theta[thIdx]*np.pi/180.) + \
                      rowIdx*np.sin(theta[thIdx]*np.pi/180)
          rhoIdx = np.nonzero(np.abs(rho-rhoVal) == np.min(np.abs(rho-rhoVal)))[0]
          H [rhoIdx[0], thIdx] += 1
  return rho, theta, H



"""Data manipulation tools.

This module provides a number of useful functions for data handling.

data_handler
------------
- `exc` -- Get items except the given list.
- `splitter` -- Split data with given size.
- `even_chunk` -- Split data into even size.
- `pair_unique` -- Get Unique pairsets.
- `df_pair_unique` -- Get unique pairsets in pandas.DataFrame.
- `merge_csv` -- Merge seperated csv type datasets into one dataset.
- `nancumsum` -- A cumulative sum function.
- `depth` -- Get dimension depth.
- `zero_padder_2d` -- Zero-padding for fixed-length inputs(2D).
- `zero_padder_3d` -- Zero-padding for fixed-length inputs(3D).

"""


from unipy.tools import data_handler

from unipy.tools.data_handler import *


__all__ = []
__all__ += data_handler.__all__



"""Docstring for ``generator``.

========================
Versatile Generators
========================
==================== =========================================================
Productivity
==============================================================================
ReusableGenerator    Reusable Generator.
re_generator         Functional API for ``ReusableGenerator``.
==================== =========================================================
==================== =========================================================
Lazy Evaluation
==============================================================================
split_generator      Split data by given size.
==================== =========================================================
==================== =========================================================
Range Generator
==============================================================================
num_fromto_generator Range number string pairs by given term.
dt_fromto_generator  Range date format string pairs by given term.
tm_fromto_generator  Range datetime format string pairs by given term.
timestamp_generator  Range timestamp string pairs by given term.
==================== =========================================================
"""


import itertools as it
import datetime as dt


__all__ = ['ReusableGenerator',
           're_generator',
           'split_generator',
           'num_fromto_generator',
           'dt_fromto_generator',
           'tm_fromto_generator',
           'timestamp_generator']


class ReusableGenerator(object):
    """Temporary Interface to re-use generator for convenience.

    Once assigned, It can be infinitely consumed
    **as long as an input generator remains un-exhausted.

    Attributes
    ----------
    _source: generator
        A source generator.

    See Also
    --------
    generator
    ``itertools.tee``

    Examples
    --------
    >>> from unipy.utils.generator import ReusableGenerator
    >>> gen = (i for i in range(10))
    >>> gen
    <generator object <genexpr> at 0x11120ebf8>
    >>> regen = ReusableGenerator(gen)
    >>> regen
    <unipy.utils.generator.ReusableGenerator object at 0x1061a97f0>
    >>> list(regen)
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    >>> list(regen)
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    >>> list(gen)  # If the source is used, copied one will be exhausted too.
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    >>> list(gen)
    []
    >>> list(regen)
    []

    """
    def __init__(self, generator):
        """__init__ method.

        Parameters
        ----------
        generator: generator
            An generator to copy.
            This original generator should not be used anywhere else.

        """
        self._copy(generator)

    def __iter__(self):
        """__iter__ method.

        """
        self._copy(self._dummy)
        return self._source.__iter__()

    def __next__(self):
        """__next__ method.

        """
        if self._source is None:
            self._copy(self._dummy)
        try:
            return self._source.__next__()
        except StopIteration:
            self._source = None
            raise

    def _copy(self, generator):
        """A private method for copy the given generator.

        """
        self._source, self._dummy = it.tee(generator)


def re_generator(generator):
    """A functional API for unipy.ReusableGenerator.

    Once assigned, It can be infinitely consumed
    **as long as an output generator is called at least one time.

    Parameters
    ----------
    generator: generator
        An generator to copy.
        This original generator should not be used anywhere else,
        until the copied one consumed at least once.

    Returns
    -------
    generator
        A generator to be used infinitely.

    See Also
    --------
    generator
    ``itertools.tee``

    Examples
    --------
    >>> from unipy.utils.generator import re_generator
    >>> gen = (i for i in range(10))
    >>> gen
    <generator object <genexpr> at 0x11120ebf8>
    >>> regen = copy_generator(gen)
    >>> regen
    <unipy.utils.generator.ReusableGenerator object at 0x1061a97f0>
    >>> list(regen)
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    >>> list(regen)
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    >>> list(gen)  # Once the copied one is used, the source will be exhausted.
    []
    >>> list(gen)
    []
    >>> list(regen)
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    >>> list(regen)
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    """
    return ReusableGenerator(generator)


def split_generator(iterable, size):

    data = iter(iterable)
    item = list(it.islice(data, size))

    while item:
        yield item
        item = list(it.islice(data, size))


def num_fromto_generator(start, end, term):
    """A range function yields pair chunks.

    It had made for time-formatting query.
    It yields a tuple of (start, start+(term-1)) pair, until start > end.

    Parameters
    ----------
    *args: int
        end or start, end[, term]
        It works like range function.

    Yields
    -------
    tuple
        A tuple of (start, start+(term-1)) pair, until start > end.

    See Also
    --------
    ``yield``

    Examples
    --------
    >>> from unipy.utils.generator import num_fromto_generator
    >>>
    >>> query = 'BETWEEN {pre} AND {nxt};'
    >>>
    >>> q_list = [query.format(pre=item[0], nxt=item[1])
    ...           for item in num_fromto_generator(1, 100, 10)]
    >>> print(q_list[0])
    BETWEEN 1 AND 10;
    >>> print(q_list[1])
    BETWEEN 11 AND 20;

    """
    pre, nxt = start, start + term - 1
    yield pre, nxt
    while nxt < end:
        pre, nxt = nxt, nxt + term
        yield pre + 1, nxt


def dt_fromto_generator(start, end, day_term, tm_format='%Y%m%d'):
    """A range function yields datetime formats by pair.

    It had made for time-formatting query.
    It yields a tuple of (start, start+(term-1)) pair, until start > end.

    Parameters
    ----------
    start: str
        start datetime like 'yyyymmdd'.

    end: str
        start datetime like 'yyyymmdd'.

    day_term: int
        term of days.

    tm_format: (default: '%Y%m%d')
        datetime format string.

    Yields
    -------
    tuple
        A tuple of (start, start+(term-1)) pair, until start > end.

    See Also
    --------
    ``yield``

    Examples
    --------
    >>> from unipy.utils.generator import dt_fromto_generator
    >>> dt_list = [item for item in
    ...            dt_fromto_generator('20170101','20170331', 10)]
    >>> dt_list[:3]
    [('20170101', '20170110'),
     ('20170111', '20170120'),
     ('20170121', '20170130')]

    """
    pre = dt.datetime.strptime(start, tm_format)
    term = dt.timedelta(days=day_term)
    nxt = pre + dt.timedelta(days=day_term-1)
    end = dt.datetime.strptime(end, tm_format)

    yield pre.strftime(tm_format), nxt.strftime(tm_format)

    while nxt < end:
        pre, nxt = nxt, nxt + term

        res_pre, res_nxt = pre + dt.timedelta(days=1), nxt
        yield res_pre.strftime(tm_format), res_nxt.strftime(tm_format)


def tm_fromto_generator(start, end, day_term,
                        tm_string=['000000', '235959'], tm_format='%Y%m%d'):
    """A range function yields datetime formats by pair.

    It had made for time-formatting query.
    It yields a tuple of (start, start+(term-1)) pair, until start > end.

    Parameters
    ----------
    start: str
        start datetime like 'yyyymmdd'.

    end: str
        start datetime like 'yyyymmdd'.

    day_term: int
        term of days.

    tm_string: list (default: ``['000000', '235959']``)
        time strings to concatenate.

    tm_format: (default: '%Y%m%d')
        datetime format string.

    Yields
    -------
    tuple
        A tuple of (start, start+(term-1)) pair, until start > end.

    See Also
    --------
    ``yield``

    Examples
    --------
    >>> from unipy.utils.generator import tm_fromto_generator
    >>> tm_list = [item for item in
    ...            tm_fromto_generator('20170101','20170331', 10)]
    >>> tm_list[:3]
    [('20170101000000', '20170110235959'),
     ('20170111000000', '20170120235959'),
     ('20170121000000', '20170130235959')]

    """
    pre = dt.datetime.strptime(start, tm_format)
    term = dt.timedelta(days=day_term)
    nxt = pre + dt.timedelta(days=day_term-1)
    end = dt.datetime.strptime(end, tm_format)

    yield (pre.strftime(tm_format) + tm_string[0],
           nxt.strftime(tm_format) + tm_string[1])

    while nxt < end:
        pre, nxt = nxt, nxt + term

        res_pre, res_nxt = pre + dt.timedelta(days=1), nxt
        yield (res_pre.strftime(tm_format) + tm_string[0],
               res_nxt.strftime(tm_format) + tm_string[1])


def timestamp_generator(*args):
    """A range function yields pair timestep strings.

    It had made for time-formatting query.
    It yields a tuple of (start, start+(term-1)) pair, until start > end.

    Parameters
    ----------
    *args: int
        end or start, end[, term]
        It works like range function.

    Yields
    -------
    tuple
        A tuple of (start, start+(term-1)) pair, until start > end.

    See Also
    --------
    ``yield``

    Examples
    --------
    >>> from unipy.utils.generator import timestamp_generator
    >>> timestamp_generator(1, 10, 2)
    <generator object timestamp_generator at 0x10f519678>
    >>> list(timestamp_generator(1, 14, 5))
    [(1, 5), (6, 10), (11, 15)]
    >>> begin, fin, period = 1, 10, 3
    >>> list(timestamp_generator(begin, fin, period))
    [(1, 3), (4, 6), (7, 9), (10, 12)]
    >>> time_sequence = timestamp_generator(begin, fin, period)
    >>> time_msg = "{start:2} to {end:2}, {term:2} days."
    >>> for time in time_sequence:
    ... b, f = time
    ... print(time_msg.format(start=b, end=f, term=period))
    ...
     1 to  3,  3 days.
     4 to  6,  3 days.
     7 to  9,  3 days.
    10 to 12,  3 days.

    """
    args_tuple = (*args, )
    args_len = len(args_tuple)
    if args_len == 1:
        start, (end, ), term = 0, args_tuple, 2
    elif args_len == 2:
        (start, end), term = args_tuple, 2
    elif args_len == 3:
        start, end, term = args_tuple
    elif args_len > 4:
        raise TypeError('expected at most 3 arguments, got %s' % args_len)

    pre, nxt = start, start + term - 1
    yield pre, nxt
    while nxt < end:
        pre, nxt = nxt, nxt + term
        yield pre + 1, nxt



"""Docstring for ``wrapper``.

========================
High-level Code Wrapper
========================
==================== =========================================================
Operation Wrapper
==============================================================================
multiprocessor       Functional wrapper for multiprocessing.
==================== =========================================================
==================== =========================================================
Interfaces
==============================================================================
uprint               Print option interface within a function.
==================== =========================================================
"""


import multiprocessing as mpr


__all__ = ['multiprocessor',
           'uprint']


def multiprocessor(func, worker=2, arg_zip=None, *args, **kwargs):
    """Use multiprocessing as a function.

    Just for convenience.

    Parameters
    ----------
    func: Function
      Any function without ``lambda``.

    worker: int (default: 2)
      A number of processes.

    arg_zip: zip (default: None)
      A ``zip`` instance.

    Returns
    -------
    list
      A list contains results of each processes.

    See Also
    --------
    ``multiprocessing.pool``

    Examples
    --------
    >>> from unipy.utils.wrapper import multiprocessor
    >>> alist = [1, 2, 3]
    >>> blist = [-1, -2, -3]
    >>> def afunc(x, y):
    ...     return x + y
    ...
    >>> multiprocessor(afunc, arg_zip=zip(alist, blist))
    [0, 0, 0]
    >>> def bfunc(x):
    ...     return x + 2
    ...
    >>> multiprocessor(bfunc, arg_zip=zip(alist))
    [3, 4, 5]

    """
    with mpr.pool.Pool(processes=worker) as pool:
        resp = pool.starmap(func, arg_zip, *args, **kwargs)

    return resp


def uprint(*args, print_ok=True, **kwargs):
    """Print option interface.

    This function is equal to ``print`` function but added ``print_ok``
    option. This allows you to control printing in a function.

    Parameters
    ----------
    *args: whatever ``print`` allows.
      It is same as ``print`` does.

    print_ok: Boolean (default: True)
      An option whether you want to print something out or not.

    arg_zip: zip (default: None)
      A ``zip`` instance.

    """
    if print_ok:
        print(*args, **kwargs)



#-*- coding: utf-8 -*-
"""
Created on 2017-01-05 20:55:26
Modified on 2017-06-26 02:36:25

@author: Young Ju Kim
"""


import os
import datetime as dt
import tarfile
import distutils
import subprocess

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages
from distutils.cmd import Command


__version__ = '0.1.12'

desc = """
This contains a number of useful objects for data manipulation & analysis.
"""

with open('README.rst', 'r', encoding='utf-8') as readme_file:
    long_desc = readme_file.read()


class SphinxCommand(Command):
    """Documentation Command"""

def initialize_options(self):
    """Set default values for options."""
    pass

def finalize_options(self):
    """Post-process options."""
    pass

def run(self):
    """Run command."""
    command = ['cd docs;make html;cd ..']
    # command.append(os.getcwd())
    self.announce(
        'Running command: %s' % str(command),
        level=distutils.log.INFO)
    subprocess.check_call(command)


with open('unipy/__version__.py', 'w') as f:
    version_py_string = '''#-*- coding: utf-8 -*-
# This file is automatically generated by 'setup.py'
"""
Created on 2017-01-05 20:53:15
Modified on {modified_time}

@author: Young Ju Kim
"""


__all__ = ['__version__']


__version__ = '{version}'

'''.format(modified_time=dt.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
           version=__version__)
    f.write(version_py_string)



def package_data_listup():

    filename = 'dataset/resources.gz/resources.tar.gz'
    tar = tarfile.open(filename)
    filelist = list(set(map(lambda x: x.split('/')[0], tar.getnames())))
    filelist.sort()
    return filelist

required_packages = [
                     'pandas >= 0.20.1',
                     'numpy >= 1.13.1',
                     'numpydoc >= 0.7.0',
                     'scipy >= 0.19.1',
                     'scikit-learn >= 0.18.0',
                     'statsmodels >= 0.8.0',
                     'matplotlib >= 2.0.2',
                     'paramiko >= 2.1.2',
                     'pandasql >= 0.7.3',
                     'seaborn >= 0.8',
                     'scikit-image >= 0.13.0',
                     'PyDrive >= 1.2.1',
                     'oauth2client >= 4.1.2',
                     'google-auth-oauthlib >= 0.2.0',
                     'pyasn1 >= 0.4.3',
                     'pyasn1-modules >= 0.2.1',
                    #'pyqt5',
                     'mglearn >= 0.1.6',
                    # 'numba >= 0.34.0',
                    # 'nomkl',  # conda
                    ]

# REQUIREMENTS.txt for [`Travis CI`, `readthedocs`]
for requirement_file in ['REQUIREMENTS.txt', 'requirements.txt']:
    with open(requirement_file, 'w') as f:
        header = '--index-url https://pypi.python.org/simple/'
        pkg_ls = '\n'.join(required_packages).replace('>=', '>=')
        f.write('\n'.join([header, pkg_ls]))

setup(name='unipy',
      version=__version__,
      description='Useful tools for Data Scientists',
      long_description=long_desc,
      url='https://unipy.readthedocs.io/en/latest/index.html',
      download_url='http://github.com/pydemia/unipy',
      author='Young Ju Kim',
      author_email='pydemia@gmail.com',
      license='MIT License',
      classifiers=[
            # How Mature: 3 - Alpha, 4 - Beta, 5 - Production/Stable
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Operating System :: OS Independent',
            'Intended Audience :: End Users/Desktop',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'Natural Language :: English',
            ],
      packages=find_packages(exclude=['contrib',
                                     #  'docs',
                                      'tests']),
      cmdclass={'documentation': SphinxCommand},
      #setup_requires=required_packages,
      install_requires=required_packages,
      zip_safe=False,
      package_data={'unipy': ['*.gz', 'dataset/resources.tar.gz']}
      )



"""Utility Objects.

This module provides a number of functions and objects for utility.

decorator
---------
- `time_profiler` -- Function running time command-line profiler.
- `time_logger` -- Function running time log profiler.
- `job_wrapper` -- Command-line line dragging tool.
- `Infix` -- Function to operator translator.
- `infix` -- Functional API for ``Infix``.

generator
---------
- `ReusableGenerator` -- Reusable Generator.
- `re_generator` -- Functional API for ``ReusableGenerator``.
- `split_generator` -- Split data by given size.
- `num_fromto_generator` -- Range number string pairs by given term.
- `dt_fromto_generator` -- Range date format string pairs by given term.
- `tm_fromto_generator` -- Range datetime format string pairs by given term.
- `timestamp_generator` -- Range timestamp string pairs by given term.

wrapper
---------
- `multiprocessor` -- Functional wrapper for multiprocessing.
- `uprint` -- Print option interface within a function.

gdrive
---------
- `gdrive_downloader` -- File downloader from Google Drive.
- `gdrive_uploader` -- File uploader to Google Drive.

"""


from unipy.utils import wrapper
from unipy.utils import decorator
from unipy.utils import generator
from unipy.utils import gdrive
# from unipy.util import remote_ipyconnector

from unipy.utils.wrapper import *
from unipy.utils.decorator import *
from unipy.utils.generator import *
from unipy.utils.gdrive import *



__all__ = []
__all__ += wrapper.__all__
__all__ += decorator.__all__
__all__ += generator.__all__
__all__ += gdrive.__all__



"""Metric Functions.

"""


import numpy as np
import pandas as pd
import statsmodels.api as sm
import itertools as it
import scipy.stats as st
from sklearn.preprocessing import PolynomialFeatures as pnf


__all__ = ['deviation',
           'vif',
           'mean_absolute_percentage_error',
           'average_absolute_deviation',
           'median_absolute_deviation',
           'calculate_interaction']


def deviation(container, method='mean', if_abs=True):

    """Deviation.

    """
    if method == 'mean':
        center = np.nanmean(container)
    elif method == 'median':
        center = np.nanmedian(container)

    resIter = map(lambda x: x - center, container)

    if if_abs:
        resIter = map(np.absolute, resIter)

    res = np.fromiter(resIter, dtype=np.float)

    return res


def vif(y, X):
    """Variance inflation factor.

    """
    assert isinstance(y, pd.Series)
    assert isinstance(X, pd.DataFrame)

    # Change input to array
    y_arr = y.values
    X_arr = X.values

    # Calculate a linear regression(Ordinary Least Square)
    reg = sm.add_constant(X_arr)
    est = sm.OLS(y_arr, reg).fit()

    # Get a R-square
    rsq = est.rsquared

    # Get a VIF
    vif = 1 / (1 - rsq)

    return vif


def mean_absolute_percentage_error(measure, predict, thresh=3.0):
    '''Mean Absolute Percentage Error.
    It is a percent of errors.
    It measures the prediction accuracy of a forecasting method in Statistics
    with the real mesured values and the predicted values, for example in trend
    estimation.
    If MAPE is 5, it means this prediction method potentially has 5% error.
    It cannot be used if there are zero values,
    because there would be a division by zero.
    '''
    mape = np.mean(np.absolute((measure - predict) / measure)) * 100

    return mape


def average_absolute_deviation(measure, predict, thresh=2):
    '''Average Absolute Deviation.
    It is ...
    It measures the prediction accuracy of a forecasting method in Statistics
    with the real mesured values and the predicted values, for example in trend
    estimation.
    If MAD is 5, it means this prediction method potentially has...
    '''
    aad = np.mean(np.absolute(measure - predict))

    return aad


def median_absolute_deviation(measure, predict, thresh=2):
    '''Median Absolute Deviation.
    It is ...
    It measures the prediction accuracy of a forecasting method in Statistics
    with the real mesured values and the predicted values, for example in trend
    estimation.
    If MAD is 5, it means this prediction method potentially has...
    '''
    mad = np.median(np.absolute(measure - predict))

    return mad


def calculate_interaction(rankTbl, pvTbl, target, ranknum=10):
    """Feature interaction calculation.

    """
    rankTop = rankTbl[:ranknum]
    interPvt = pvTbl[rankTop['var_name']]
    interAct = pnf(degree=2, interaction_only=True)

    interTbl = pd.DataFrame(interAct.fit_transform(interPvt),
                            index=interPvt.index).iloc[:, 1:]
    rankTop_col = list(rankTop['var_name'])
    interAct_col = list(map(' xx '.join,
                        list(it.combinations(rankTop['var_name'], 2))))
    interTbl.columns = rankTop_col + interAct_col

    # Generate a Result Table
    col = ['slope', 'intercept', 'corr_coef', 'p_value', 'std_err']
    ind = interTbl.columns
    regMatrix = pd.DataFrame(index=ind, columns=col)

    # Regression
    Y = pvTbl[target]
    for _ in range(interTbl.shape[1]):
        x = interTbl.ix[:, _]
        regMatrix.iloc[_, ] = st.linregress(x, Y)

    regMatrix['abs_corr_coef'] = abs(regMatrix['corr_coef'])
    regMatrix.sort_values(by='p_value', ascending=True, inplace=True)

    rank = regMatrix[(regMatrix['p_value'] < .01) &
                     (regMatrix['abs_corr_coef'] >= .3)]

    rank = rank.reset_index()
    rank['inter_name'] = rank['index']
    rank = rank[rank['inter_name'].str.find(' xx ') != -1]
    rank['rank'] = range(1, len(rank) + 1)

    rankCol = ['rank', 'inter_name', 'p_value',
               'corr_coef', 'abs_corr_coef',
               'std_err', 'slope', 'intercept']
    rank = rank[rankCol]

    return rank, regMatrix, interTbl



# -*- coding: utf-8 -*-
"""
Created on Thu Jan  5 20:55:26 2017

@author: Young Ju Kim

"""


if __name__ == '__main__':
    import numpy as np
    import unipy.dataset.api as dm
    from unipy.tools.data_handler import splitter, even_chunk
    from unipy.tools.data_handler import pair_unique, df_pair_unique
    from unipy.tools.data_handler import (merge_csv, nancumsum,
                                          timestamp_generator)
    from unipy.tools.data_handler import depth
    from unipy.tools.data_handler import zero_padder_2d, zero_padder_3d
    from unipy.tools.data_handler import ReusableGenerator, re_generator


    dm.init()

    # Function: splitter
    splitter(list(range(10)), how='equal', size=3)
    splitter(list(range(10)), how='remaining', size=3)

    # Function: even_chunk
    data = list(range(7))  # list, 1D
    chunked_gen = even_chunk(data, 3)
    next(chunked_gen)
    chunked = list(even_chunk(data, 3))

    data = np.arange(30).reshape(-1, 3)  # np.ndarray, 2D
    chunked_gen = even_chunk(data, 4)
    next(chunked_gen)

    # Function: pair_unique
    data = dm.load('titanic')
    pair_unique(data.iloc[:, 0], data.iloc[:, 1])

    # Function: df_pair_unique
    data = dm.load('titanic')
    df_pair_unique(data, ['Class', 'Sex'])
    df_pair_unique(data, ['Class', 'Sex'], to_frame=True)

    # Function: merge_csv
    data = dm.load('titanic')
    data.head(9)
    data.iloc[:2, :].to_csv('tmp1.csv', header=True, index=False)
    data.iloc[2:4, :].to_csv('tmp2.csv', header=True, index=False)
    data.iloc[4:9, :].to_csv('tmp3.csv', header=True, index=False)
    merged = merge_csv('./')
    print(merged)

    # Function: nancumsum
    tmp = [1, 2, 4]
    list(nancumsum(tmp))

    # Function: timestamp_generator
    begin, fin, period = 1, 10, 3
    list(timestamp_generator(begin, fin, period))
    time_sequence = timestamp_generator(begin, fin, period)
    time_msg = "{start:2} to {end:2}, {term:2} days."
    for time in time_sequence:
        b, f = time
        print(time_msg.format(start=b, end=f, term=period))

    # Function: depth
    tmp = [(1, 3), (4, 6), (7, 9), (10, 12)]
    depth(tmp)

    tmp3d = [[np.arange(i) + i for i in range(2, j)] for j in range(5, 10)]
    depth(tmp3d)

    tmp3d_dict = [{'key' + str(i): np.arange(i) + i for i in range(2, j)}
                  for j in range(5, 10)]
    depth(tmp3d_dict)

    # Function: zero_padder_2d
    tmp = [np.arange(i) + i for i in range(2, 5)]
    zero_padder_2d(tmp, max_len=5, method='forward')

    # Function: zero_padder_3d
    tmp3d = [np.arange(i * 2).reshape(-1, 2) for i in range(1, 5)]
    zero_padder_3d(tmp3d)

    # Class: ReusableGenerator
    gen = (i for i in range(10))
    regen = ReusableGenerator(gen)
    regen
    list(regen)
    list(regen)
    list(gen)
    list(regen)

    # Function: re_generator
    gen = (i for i in range(10))
    regen = re_generator(gen)
    regen
    list(regen)
    list(regen)
    list(gen)
    list(gen)
    list(gen)
    list(regen)
    list(regen)
